@@ -97,12 +97,12 @@ PR_END_EXTERN_C
 nsXPrintContext::nsXPrintContext() :
   mXlibRgbHandle(nsnull),
   mPDisplay(nsnull),
-  mPContext(None),
   mScreen(nsnull),
   mVisual(nsnull),
   mDrawable(None),
   mGC(nsnull),
   mDepth(0),
+  mPContext(None),
   mJobStarted(PR_FALSE),
   mIsGrayscale(PR_FALSE), /* default is color output */
   mIsAPrinter(PR_TRUE),   /* default destination is printer */
@@ -111,7 +111,6 @@ nsXPrintContext::nsXPrintContext() :
   mPrintResolution(0L),
   mContext(nsnull)
 {
-  NS_INIT_REFCNT();
   PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("nsXPrintContext::nsXPrintContext()\n"));
 }
 
@@ -206,7 +205,7 @@ nsXPrintContext::Init(nsDeviceContextXp *dc, nsIDeviceContextSpecXp *aSpec)
 
   XlibRgbArgs xargs;
   memset(&xargs, 0, sizeof(xargs));
-  xargs.handle_name           = "xprint";
+  xargs.handle_name           = nsnull;
   xargs.disallow_image_tiling = True; /* XlibRGB's "Image tiling"-hack is 
                                        * incompatible to Xprint API */
   
@@ -389,7 +388,7 @@ nsresult nsXPrintContext::SetMediumSize(const char *aPaperName)
   char *s;
   
   /* Did we get a tray name and paper name (e.g. "manual/din-a4") ? */
-  if (s = strchr(paper_name, '/'))
+  if ((s = strchr(paper_name, '/')) != nsnull)
   {
     const char *tray_name;
     *s = '\0';
@@ -536,7 +535,7 @@ nsXPrintContext::SetupPrintContext(nsIDeviceContextSpecXp *aSpec)
 
     PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("print to file '%s'\n", XPU_NULLXSTR(mPrintFile)));
     
-    if( (mPrintFile == nsnull) || (strlen(mPrintFile) == 0) )
+    if(!mPrintFile || !*mPrintFile)
       return NS_ERROR_GFX_PRINTER_COULD_NOT_OPEN_FILE;
   }
 
@@ -685,7 +684,7 @@ nsXPrintContext::SetResolution( void )
 }
   
 NS_IMETHODIMP
-nsXPrintContext::BeginDocument( PRUnichar *aTitle )
+nsXPrintContext::BeginDocument( PRUnichar * aTitle, PRUnichar* aPrintToFileName, PRInt32 aStartPage, PRInt32 aEndPage )
 {
   PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("nsXPrintContext::BeginDocument(aTitle='%s')\n", ((aTitle)?(NS_ConvertUCS2toUTF8(aTitle).get()):("<NULL>"))));
   
@@ -960,13 +959,27 @@ nsXPrintContext::DrawImage(xGC *xgc, nsIImage *aImage,
    * Y-axis - therefore we want to get the "best"(=smallest job size) out
    * of the scaling factor...).
    */
-  scalingFactor *= (scale_x < scale_y)?(scale_x):(scale_y);
+  scalingFactor *= PR_MIN(scale_x, scale_y);
   
   /* Adjust destination size to the match the scaling factor */
   imageResolution = long(   double(mPrintResolution) * scalingFactor);
   aDWidth_scaled  = PRInt32(double(aDWidth)          * scalingFactor);
   aDHeight_scaled = PRInt32(double(aDHeight)         * scalingFactor);
 
+  /* Check if we scaled the image to zero width/height - if "yes" then use the
+   * smaller scaling factor and try again ... */
+  if( (aDWidth_scaled <= 0) || (aDHeight_scaled <= 0) )
+  {
+    /* Chose smaller scaling factor (e.g. the reverse of what we're doing above...) */
+    scalingFactor = 1.0 / pixelscale; /* Revert to original value */
+    scalingFactor *= PR_MAX(scale_x, scale_y);
+
+    /* Adjust destination size to the match the (new) scaling factor */
+    imageResolution = long(   double(mPrintResolution) * scalingFactor);
+    aDWidth_scaled  = PRInt32(double(aDWidth)          * scalingFactor);
+    aDHeight_scaled = PRInt32(double(aDHeight)         * scalingFactor);
+  }
+
   /* Image scaled to 0 width/height ? */
   NS_ASSERTION(!((aDWidth_scaled <= 0) || (aDHeight_scaled <= 0)),
                "Image scaled to zero width/height");
@@ -1047,6 +1060,13 @@ nsXPrintContext::DrawImageBitsScaled(xGC *xgc, nsIImage *aImage,
   PRInt32  aSrcHeight    = aImage->GetHeight();
   PRUint8 *composed_bits = nsnull;
 
+ /* image data might not be available (ex: spacer image) */
+  if (!image_bits)
+  {
+    aImage->UnlockImagePixels(PR_FALSE);
+    return NS_OK;
+  }
+ 
   // Use client-side alpha image composing - plain X11 can only do 1bit alpha 
   // stuff - this method adds 8bit alpha support, too...
   if( alphaBits != nsnull )
@@ -1125,6 +1145,13 @@ nsXPrintContext::DrawImage(xGC *xgc, nsIImage *aImage,
   PRUint8 *image_bits    = aImage->GetBits();
   PRUint8 *composed_bits = nsnull;
   PRInt32  row_bytes     = aImage->GetLineStride();
+
+  /* image data might not be available (ex: spacer image) */
+  if (!image_bits)
+  {
+    aImage->UnlockImagePixels(PR_FALSE);
+    return NS_OK;
+  }
   
   // Use client-side alpha image composing - plain X11 can only do 1bit alpha
   // stuff - this method adds 8bit alpha support, too...
