@@ -40,8 +40,10 @@
 #include "nsISupports.h"
 #include "nsColor.h"
 #include "nsCoord.h"
-#include "nsAWritableString.h"
+#include "nsAString.h"
 #include "nsIRequest.h"
+#include "nsCompatibility.h"
+#include "nsCOMPtr.h"
 #ifdef IBMBIDI
 class nsBidiPresUtils;
 #endif // IBMBIDI
@@ -60,15 +62,18 @@ class nsIImage;
 class nsILinkHandler;
 class nsIPresShell;
 class nsIPref;
-class nsIStyleContext;
+class nsStyleContext;
 class nsIAtom;
 class nsString;
 class nsIEventStateManager;
 class nsIURI;
 class nsILookAndFeel;
+class nsIIOService;
 class nsICSSPseudoComparator;
 class nsILanguageAtom;
 class nsITheme;
+struct nsStyleStruct;
+struct nsStyleBackground;
 
 #ifdef MOZ_REFLOW_PERF
 class nsIRenderingContext;
@@ -78,17 +83,6 @@ class nsIRenderingContext;
 { 0x0a5d12e0, 0x944e, 0x11d1, \
   {0x93, 0x23, 0x00, 0x80, 0x5f, 0x8a, 0xdd, 0x32} }
 
-enum nsCompatibility {
-  eCompatibility_Standard   = 1,
-  eCompatibility_NavQuirks  = 2
-};
-
-enum nsWidgetRendering {
-  eWidgetRendering_Native   = 1,
-  eWidgetRendering_Gfx      = 2,
-  eWidgetRendering_PartialGfx = 3
-};
-
 enum nsWidgetType {
   eWidgetType_Button  	= 1,
   eWidgetType_Checkbox	= 2,
@@ -120,7 +114,7 @@ const PRUint8 kPresContext_DefaultFixedFont_ID    = 0x01; // kGenericFont_moz_fi
 // objects that provide an outer context for a presentation shell.
 class nsIPresContext : public nsISupports {
 public:
-  static const nsIID& GetIID() { static nsIID iid = NS_IPRESCONTEXT_IID; return iid; }
+  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IPRESCONTEXT_IID)
 
   /**
    * Initialize the presentation context from a particular device.
@@ -147,12 +141,6 @@ public:
   NS_IMETHOD GetCompatibilityMode(nsCompatibility* aModeResult) = 0;
   NS_IMETHOD SetCompatibilityMode(nsCompatibility aMode) = 0;
 
-  /**
-   * Access the widget rendering mode for this context
-   */
-  NS_IMETHOD GetWidgetRenderingMode(nsWidgetRendering* aModeResult) = 0;
-  NS_IMETHOD SetWidgetRenderingMode(nsWidgetRendering aMode) = 0;
-
   /**
    * Access the image animation mode for this context
    */
@@ -165,10 +153,15 @@ public:
   NS_IMETHOD GetImageLoadFlags(nsLoadFlags& aLoadFlags) = 0;
 
   /**
-   * Get look and feel object
+   * Get cached look and feel service.
    */
   NS_IMETHOD GetLookAndFeel(nsILookAndFeel** aLookAndFeel) = 0;
 
+  /**
+   * Get cached IO service.
+   */
+  NS_IMETHOD GetIOService(nsIIOService** aIOService) = 0;
+
   /** 
    * Get base url for presentation
    */
@@ -188,10 +181,9 @@ public:
    * Resolve style for the given piece of content that will be a child
    * of the aParentContext. Don't use this for pseudo frames.
    */
-  NS_IMETHOD ResolveStyleContextFor(nsIContent* aContent,
-                                    nsIStyleContext* aParentContext,
-                                    PRBool aForceUnique,
-                                    nsIStyleContext** aResult) = 0;
+  virtual already_AddRefed<nsStyleContext>
+  ResolveStyleContextFor(nsIContent* aContent,
+                         nsStyleContext* aParentContext) = 0;
 
   /**
    * Resolve style for a non-element content node (i.e., one that is
@@ -205,21 +197,19 @@ public:
    * represents everything except the first letter.)
    *
    */
-  NS_IMETHOD ResolveStyleContextForNonElement(
-                                    nsIStyleContext* aParentContext,
-                                    PRBool aForceUnique,
-                                    nsIStyleContext** aResult) = 0;
+  virtual already_AddRefed<nsStyleContext>
+  ResolveStyleContextForNonElement(nsStyleContext* aParentContext) = 0;
 
   /**
    * Resolve style for a pseudo frame within the given aParentContent & aParentContext.
    * The tag should be lowercase and inclue the colon.
    * ie: NS_NewAtom(":first-line");
    */
-  NS_IMETHOD ResolvePseudoStyleContextFor(nsIContent* aParentContent,
-                                          nsIAtom* aPseudoTag,
-                                          nsIStyleContext* aParentContext,
-                                          PRBool aForceUnique,
-                                          nsIStyleContext** aResult) = 0;
+
+  virtual already_AddRefed<nsStyleContext>
+  ResolvePseudoStyleContextFor(nsIContent* aParentContent,
+                               nsIAtom* aPseudoTag,
+                               nsStyleContext* aParentContext) = 0;
 
   /**
    * Resolve style for a pseudo frame within the given aParentContent & aParentContext.
@@ -229,12 +219,11 @@ public:
    * Instead of matching solely on aPseudoTag, a comparator function can be
    * passed in to test.
    */
-  NS_IMETHOD ResolvePseudoStyleWithComparator(nsIContent* aParentContent,
-                                              nsIAtom* aPseudoTag,
-                                              nsIStyleContext* aParentContext,
-                                              PRBool aForceUnique,
-                                              nsICSSPseudoComparator* aComparator,
-                                              nsIStyleContext** aResult) = 0;
+  virtual already_AddRefed<nsStyleContext>
+  ResolvePseudoStyleWithComparator(nsIContent* aParentContent,
+                                   nsIAtom* aPseudoTag,
+                                   nsStyleContext* aParentContext,
+                                   nsICSSPseudoComparator* aComparator) = 0;
 
   /**
    * Probe style for a pseudo frame within the given aParentContent & aParentContext.
@@ -242,18 +231,24 @@ public:
    * The tag should be lowercase and inclue the colon.
    * ie: NS_NewAtom(":first-line");
    */
-  NS_IMETHOD ProbePseudoStyleContextFor(nsIContent* aParentContent,
-                                        nsIAtom* aPseudoTag,
-                                        nsIStyleContext* aParentContext,
-                                        PRBool aForceUnique,
-                                        nsIStyleContext** aResult) = 0;
+  virtual already_AddRefed<nsStyleContext>
+  ProbePseudoStyleContextFor(nsIContent* aParentContent,
+                             nsIAtom* aPseudoTag,
+                             nsStyleContext* aParentContext) = 0;
+ 
+   /**
+    * Resolve a new style context for a content node and return the URL
+    * for its XBL binding, or the empty string if it has no binding
+    * specified in CSS.
+    */
+   NS_IMETHOD GetXBLBindingURL(nsIContent* aContent, nsAString& aResult) = 0;
 
   /** 
    * For a given frame tree, get a new style context that is the equivalent
    * but within a new parent
    */
   NS_IMETHOD ReParentStyleContext(nsIFrame* aFrame, 
-                                  nsIStyleContext* aNewParentContext) = 0;
+                                  nsStyleContext* aNewParentContext) = 0;
 
 
   NS_IMETHOD AllocateFromShell(size_t aSize, void** aResult) = 0;
@@ -294,10 +289,6 @@ public:
    */
   NS_IMETHOD GetDefaultColor(nscolor* aColor) = 0;
   NS_IMETHOD GetDefaultBackgroundColor(nscolor* aColor) = 0;
-  NS_IMETHOD GetDefaultBackgroundImage(nsString& aImage) = 0;
-  NS_IMETHOD GetDefaultBackgroundImageRepeat(PRUint8* aRepeat) = 0;
-  NS_IMETHOD GetDefaultBackgroundImageOffset(nscoord* aX, nscoord* aY) = 0;
-  NS_IMETHOD GetDefaultBackgroundImageAttachment(PRUint8* aRepeat) = 0;
   NS_IMETHOD GetDefaultLinkColor(nscolor* aColor) = 0;
   NS_IMETHOD GetDefaultVisitedLinkColor(nscolor* aColor) = 0;
   NS_IMETHOD GetFocusBackgroundColor(nscolor* aColor) = 0;
@@ -309,10 +300,6 @@ public:
 
   NS_IMETHOD SetDefaultColor(nscolor aColor) = 0;
   NS_IMETHOD SetDefaultBackgroundColor(nscolor aColor) = 0;
-  NS_IMETHOD SetDefaultBackgroundImage(const nsString& aImage) = 0;
-  NS_IMETHOD SetDefaultBackgroundImageRepeat(PRUint8 aRepeat) = 0;
-  NS_IMETHOD SetDefaultBackgroundImageOffset(nscoord aX, nscoord aY) = 0;
-  NS_IMETHOD SetDefaultBackgroundImageAttachment(PRUint8 aRepeat) = 0;
   NS_IMETHOD SetDefaultLinkColor(nscolor aColor) = 0;
   NS_IMETHOD SetDefaultVisitedLinkColor(nscolor aColor) = 0;
 
@@ -374,7 +361,7 @@ public:
   NS_IMETHOD GetPaginatedScrolling(PRBool* aResult) = 0;
 
   /**
-   * Gets the rect for the page Dimimensions, 
+   * Gets the rect for the page dimensions,
    * this includes X,Y Offsets which are used to determine 
    * the inclusion of margins
    * Also, indicates whether the size has been overridden
@@ -486,20 +473,20 @@ public:
   /**
    * Get the Bidi options for the presentation context
    */  
-  NS_IMETHOD GetBidi(PRUint32* aBidiOptions) = 0;
+  NS_IMETHOD GetBidi(PRUint32* aBidiOptions) const = 0;
 //ahmed
 
   /**
    * Check for Bidi text mode and direction
    * @return aResult == TRUE if the text mode is visual and the direction is right-to-left
    */
-  NS_IMETHOD IsVisRTL(PRBool &aResult) = 0;
+  NS_IMETHOD IsVisRTL(PRBool &aResult) const = 0;
 
   /**
    * Check for Arabic encoding
    * @return aResult == TRUE if the document encoding is an Arabic codepage
    */
-  NS_IMETHOD IsArabicEncoding(PRBool &aResult) = 0;
+  NS_IMETHOD IsArabicEncoding(PRBool &aResult) const = 0;
 
   /**
    * Set the Bidi capabilities of the system
@@ -516,7 +503,7 @@ public:
   /**
    * Get the document charset
    */
-  NS_IMETHOD GetBidiCharset(nsAWritableString &aCharSet) = 0;
+  NS_IMETHOD GetBidiCharset(nsAString &aCharSet) const = 0;
 
 
 #endif // IBMBIDI
@@ -540,6 +527,21 @@ public:
    */
   NS_IMETHOD ThemeChanged() = 0;
 
+  /*
+   * Notify the pres context that a system color has changed
+   */
+  NS_IMETHOD SysColorChanged() = 0;
+
+  /*
+   * Fill in an nsStyleBackground to be used to paint the background for an
+   * element.  This applies the rules for propagating backgrounds between
+   * BODY, the root element, and the canvas.
+   */
+  NS_IMETHOD FindFrameBackground(nsIFrame* aFrame,
+                                 const nsStyleBackground** aBackground,
+                                 PRBool* aIsCanavs,
+                                 PRBool* aFoundBackground) = 0;
+
 #ifdef MOZ_REFLOW_PERF
   NS_IMETHOD CountReflows(const char * aName, PRUint32 aType, nsIFrame * aFrame) = 0;
   NS_IMETHOD PaintCount(const char * aName, nsIRenderingContext* aRendingContext, nsIFrame * aFrame, PRUint32 aColor) = 0;
@@ -553,8 +555,8 @@ public:
 
 // Factory method to create a "galley" presentation context (galley is
 // a kind of view that has no limit to the size of a page)
-extern NS_EXPORT nsresult
-  NS_NewGalleyContext(nsIPresContext** aInstancePtrResult);
+nsresult
+NS_NewGalleyContext(nsIPresContext** aInstancePtrResult);
 
 #ifdef MOZ_REFLOW_PERF
 
