@@ -31,6 +31,7 @@
 #include "nsBidiUtils.h"
 #include "bidicattable.h"
 #include "symmtable.h"
+#include "nsCRT.h"
 
 static nsCharType ebc2ucd[15] = {
   eCharType_OtherNeutral, /* Placeholder -- there will never be a 0 index value */
@@ -852,7 +853,7 @@ void nsBidi::ResolveImplicitLevels(PRInt32 aStart, PRInt32 aLimit,
 
   /* initialize: current at aSOR, next at aStart (it is aStart<aLimit) */
   next=aStart;
-  dirProp=lastStrong=aSOR;
+  beforeNeutral=dirProp=lastStrong=aSOR;
   nextDirProp=dirProps[next];
   historyOfEN=0;
 
@@ -1706,8 +1707,7 @@ void nsBidi::ReorderLine(nsBidiLevel aMinLevel, nsBidiLevel aMaxLevel)
 {
   Run *runs;
   nsBidiLevel *levels;
-  PRInt32 firstRun, endRun, limitRun, runCount,
-  temp, trailingWSStart=mTrailingWSStart;
+  PRInt32 firstRun, endRun, limitRun, runCount, temp;
 
   /* nothing to do? */
   if(aMaxLevel<=(aMinLevel|1)) {
@@ -2255,7 +2255,7 @@ PRInt32 nsBidi::doWriteReverse(const PRUnichar *src, PRInt32 srcLength,
         j=srcLength;
         if(options&NSBIDI_DO_MIRRORING) {
           /* mirror only the base character */
-          c = SymmSwap((PRUnichar)c);
+          c = SymmSwap(c);
 
           PRInt32 k=0;
           UTF_APPEND_CHAR_UNSAFE(dest, k, c);
@@ -2305,13 +2305,13 @@ PRBool nsBidi::IsBidiCategory(PRUnichar aChar, eBidiCategory aBidiCategory)
   return (GetBidiCategory(aChar) == aBidiCategory);
 }
 
-#define LRM_CHAR 0x200e
+#define ZWNJ 0x200c
 PRBool nsBidi::IsBidiControl(PRUnichar aChar)
 {
   // This method is used when stripping Bidi control characters for
-  // display, so it will return TRUE for LRM and RLM as well as the
-  // characters with category eBidiCat_CC
-  return (eBidiCat_CC == GetBidiCat(aChar) || ((aChar)&0xfffe)==LRM_CHAR);
+  // display, so it will return TRUE for LRM, RLM, ZWJ and ZWNJ as
+  // well as the characters with category eBidiCat_CC
+  return (eBidiCat_CC == GetBidiCat(aChar) || ((aChar)&0xfffc)==ZWNJ);
 }
 
 nsCharType nsBidi::GetCharType(PRUnichar aChar)
@@ -2334,7 +2334,7 @@ nsCharType nsBidi::GetCharType(PRUnichar aChar)
   return oResult;
 }
 
-PRUnichar nsBidi::SymmSwap(PRUnichar aChar)
+PRUint32 nsBidi::SymmSwap(PRUint32 aChar)
 {
   return Mirrored(aChar);
 }
