@@ -43,9 +43,9 @@
 #include "nsAbBaseCID.h"
 #include "nsDirPrefs.h"
 #include "nsIAddrBookSession.h"
-#include "nsIAddrDatabase.h"
+#include "nsAddrDatabase.h"
 #include "nsIOutputStream.h"
-#include "nsIFileStreams.h"
+#include "nsNetUtil.h"
 #include "msgCore.h"
 #include "nsIImportService.h"
 #include "nsIStringBundle.h"
@@ -72,6 +72,8 @@
 #include "nsSpecialSystemDirectory.h"
 #include "nsIFilePicker.h"
 #include "nsIPref.h"
+#include "nsIPrefService.h"
+#include "nsIPrefBranch.h"
 #include "nsVoidArray.h"
 #include "nsIAbCard.h"
 #include "nsIAbMDBCard.h"
@@ -96,75 +98,13 @@
 // see bugs bug #116692 and #118454
 #define MOZ_AB_OBJECTCLASS "mozillaAbPersonObsolete"
 
-// for now, these should be in the same order as they are in the import code
-// see importMsgProperties and nsImportStringBundle.h
-
-#define   EXPORT_ATTRIBUTES_TABLE_COUNT      38
-
-struct ExportAttributesTableStruct
-{
-    const char* abColName;
-    const char* ldapPropertyName;
-    PRBool includeForPlainText;
-};
-
-// XXX todo, merge with what's in nsAbLDAPProperties.cpp, so we can
-// use this for LDAP and LDIF export
-// XXX todo what about _AimScreenName, or other generic columns?
-//
-// here's how we're coming up with the ldapPropertyName values
-// if they are specified in RFC 2798, use them
-// else use the 4.x LDIF attribute names (for example, "xmozillanickname"
-// as we want to allow export from mozilla back to 4.x, and other apps
-// are probably out there that can handle 4.x LDIF)
-// else use the MOZ_AB_LDIF_PREFIX prefix, see nsIAddrDatabase.idl
-static ExportAttributesTableStruct EXPORT_ATTRIBUTES_TABLE[] = { 
-  {kFirstNameColumn, "givenName", PR_TRUE},
-  {kLastNameColumn, "sn", PR_TRUE},
-  {kDisplayNameColumn, "cn", PR_TRUE},
-  {kNicknameColumn, "xmozillanickname", PR_TRUE},
-  {kPriEmailColumn, "mail", PR_TRUE},
-  {k2ndEmailColumn, MOZ_AB_LDIF_PREFIX "SecondEmail", PR_TRUE},
-  {kPreferMailFormatColumn, "xmozillausehtmlmail", PR_FALSE},
-  {kLastModifiedDateColumn, "modifytimestamp", PR_FALSE},
-  {kWorkPhoneColumn, "telephoneNumber", PR_TRUE}, 
-  {kHomePhoneColumn, "homePhone", PR_TRUE},
-  {kFaxColumn, "facsimileTelephoneNumber", PR_TRUE},
-  {kPagerColumn, "pager", PR_TRUE},
-  {kCellularColumn, "mobile", PR_TRUE},
-  {kHomeAddressColumn, "homePostalAddress", PR_TRUE},
-  {kHomeAddress2Column, MOZ_AB_LDIF_PREFIX "HomePostalAddress2", PR_TRUE},
-  {kHomeCityColumn, MOZ_AB_LDIF_PREFIX "HomeLocalityName", PR_TRUE},
-  {kHomeStateColumn, MOZ_AB_LDIF_PREFIX "HomeState", PR_TRUE},
-  {kHomeZipCodeColumn, MOZ_AB_LDIF_PREFIX "HomePostalCode", PR_TRUE},
-  {kHomeCountryColumn, MOZ_AB_LDIF_PREFIX "HomeCountryName", PR_TRUE},
-  {kWorkAddressColumn, "postalAddress", PR_TRUE},
-  {kWorkAddress2Column, MOZ_AB_LDIF_PREFIX "PostalAddress2", PR_TRUE}, 
-  {kWorkCityColumn, "l", PR_TRUE}, 
-  {kWorkStateColumn, "st", PR_TRUE}, 
-  {kWorkZipCodeColumn, "postalCode", PR_TRUE}, 
-  {kWorkCountryColumn, "c", PR_TRUE}, 
-  {kJobTitleColumn, "title", PR_TRUE},
-  {kDepartmentColumn, "ou", PR_TRUE},
-  {kCompanyColumn, "o", PR_TRUE},
-  {kWebPage1Column, "workurl", PR_TRUE},
-  {kWebPage2Column, "homeurl", PR_TRUE},
-  {kBirthYearColumn, nsnull, PR_TRUE}, // unused for now
-  {kBirthMonthColumn, nsnull, PR_TRUE}, // unused for now
-  {kBirthDayColumn, nsnull, PR_TRUE}, // unused for now
-  {kCustom1Column, "custom1", PR_TRUE},
-  {kCustom2Column, "custom2", PR_TRUE},
-  {kCustom3Column, "custom3", PR_TRUE},
-  {kCustom4Column, "custom4", PR_TRUE},
-  {kNotesColumn, "description", PR_TRUE},
-};
 
 //
 // nsAddressBook
 //
 nsAddressBook::nsAddressBook()
 {
-  NS_INIT_ISUPPORTS();
+  mDocShell = nsnull;
 }
 
 nsAddressBook::~nsAddressBook()
@@ -190,7 +130,8 @@ NS_IMETHODIMP nsAddressBook::NewAddressBook(nsIAbDirectoryProperties *aPropertie
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIRDFResource> parentResource;
-    rv = rdfService->GetResource(kAllDirectoryRoot, getter_AddRefs(parentResource));
+    rv = rdfService->GetResource(NS_LITERAL_CSTRING(kAllDirectoryRoot),
+                                 getter_AddRefs(parentResource));
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIAbDirectory> parentDir = do_QueryInterface(parentResource, &rv);
@@ -200,18 +141,46 @@ NS_IMETHODIMP nsAddressBook::NewAddressBook(nsIAbDirectoryProperties *aPropertie
     return rv;
 }
 
-NS_IMETHODIMP nsAddressBook::DeleteAddressBooks
-(nsIRDFCompositeDataSource* db, nsISupportsArray *parentDir, nsISupportsArray *aResourceArray)
+NS_IMETHODIMP nsAddressBook::ModifyAddressBook
+(nsIRDFDataSource* aDS, nsIAbDirectory *aParentDir, nsIAbDirectory *aDirectory, nsIAbDirectoryProperties *aProperties)
 {
-  NS_ENSURE_ARG_POINTER(db);
-  NS_ENSURE_ARG_POINTER(parentDir);
-  NS_ENSURE_ARG_POINTER(aResourceArray);
+  NS_ENSURE_ARG_POINTER(aDS);
+  NS_ENSURE_ARG_POINTER(aParentDir);
+  NS_ENSURE_ARG_POINTER(aDirectory);
+  NS_ENSURE_ARG_POINTER(aProperties);
 
-    return DoCommand(db, NC_RDF_DELETE, parentDir, aResourceArray);
+  nsresult rv;
+  nsCOMPtr<nsISupportsArray> parentArray (do_CreateInstance(NS_SUPPORTSARRAY_CONTRACTID, &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsISupportsArray> resourceElement (do_CreateInstance(NS_SUPPORTSARRAY_CONTRACTID, &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsISupportsArray> resourceArray (do_CreateInstance(NS_SUPPORTSARRAY_CONTRACTID, &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  parentArray->AppendElement(aParentDir);
+
+  nsCOMPtr<nsIRDFResource> dirSource(do_QueryInterface(aDirectory, &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  resourceElement->AppendElement(dirSource);
+  resourceElement->AppendElement(aProperties);
+  resourceArray->AppendElement(resourceElement);
+
+  return DoCommand(aDS, NS_LITERAL_CSTRING(NC_RDF_MODIFY), parentArray, resourceArray);
 }
 
-nsresult nsAddressBook::DoCommand(nsIRDFCompositeDataSource* db,
-                                  const char *command,
+NS_IMETHODIMP nsAddressBook::DeleteAddressBooks
+(nsIRDFDataSource* aDS, nsISupportsArray *aParentDir, nsISupportsArray *aResourceArray)
+{
+  NS_ENSURE_ARG_POINTER(aDS);
+  NS_ENSURE_ARG_POINTER(aParentDir);
+  NS_ENSURE_ARG_POINTER(aResourceArray);
+  
+  return DoCommand(aDS, NS_LITERAL_CSTRING(NC_RDF_DELETE), aParentDir, aResourceArray);
+}
+
+nsresult nsAddressBook::DoCommand(nsIRDFDataSource* db,
+                                  const nsACString& command,
                                   nsISupportsArray *srcArray,
                                   nsISupportsArray *argumentArray)
 {
@@ -288,11 +257,10 @@ NS_IMETHODIMP nsAddressBook::GetAbDatabaseFromURI(const char *aURI, nsIAddrDatab
     do_GetService(NS_ADDRDATABASE_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv,rv);
 
-  rv = addrDBFactory->Open(dbPath, PR_TRUE, aDB, PR_TRUE);
-  NS_ENSURE_SUCCESS(rv,rv);
-
+  rv = addrDBFactory->Open(dbPath, PR_TRUE /* create */, aDB, PR_TRUE);
   delete dbPath;
-    
+  NS_ENSURE_SUCCESS(rv,rv);
+  
   return NS_OK;
 }
 
@@ -330,51 +298,39 @@ nsresult nsAddressBook::GetAbDatabaseFromFile(char* pDbFile, nsIAddrDatabase **d
     return NS_OK;
 }
 
-NS_IMETHODIMP nsAddressBook::MailListNameExistsInDB(const PRUnichar *name, const char *URI, PRBool *exist)
-{
-    *exist = PR_FALSE;
-
-    nsCOMPtr<nsIAddrDatabase> database;
-    nsresult rv = GetAbDatabaseFromURI(URI, getter_AddRefs(database));                
-    if (NS_SUCCEEDED(rv) && database)
-        database->FindMailListbyUnicodeName(name, exist);
-    return NS_OK;
-}
-
-//check for all address book
 NS_IMETHODIMP nsAddressBook::MailListNameExists(const PRUnichar *name, PRBool *exist)
 {
-    *exist = PR_FALSE;
-    nsVoidArray* pDirectories = DIR_GetDirectories();
-    if (pDirectories)
+  *exist = PR_FALSE;
+  nsVoidArray* pDirectories = DIR_GetDirectories();
+  if (pDirectories)
+  {
+    PRInt32 count = pDirectories->Count();
+    /* check: only show personal address book for now */
+    /* not showing 4.x address book unitl we have the converting done */
+    PRInt32 i;
+    for (i = 0; i < count; i++)
     {
-        PRInt32 count = pDirectories->Count();
-        /* check: only show personal address book for now */
-        /* not showing 4.x address book unitl we have the converting done */
-        PRInt32 i;
-        for (i = 0; i < count; i++)
+      DIR_Server *server = (DIR_Server *)pDirectories->ElementAt(i);
+      if (server->dirType == PABDirectory)
+      {
+        /* check: this is a 4.x file, remove when conversion is done */
+        PRUint32 fileNameLen = strlen(server->fileName);
+        if ((fileNameLen > kABFileName_PreviousSuffixLen) && 
+          strcmp(server->fileName + fileNameLen - kABFileName_PreviousSuffixLen, kABFileName_PreviousSuffix) == 0)
+          continue;
+        
+        nsCOMPtr<nsIAddrDatabase> database;
+        nsresult rv = GetAbDatabaseFromFile(server->fileName, getter_AddRefs(database));                
+        if (NS_SUCCEEDED(rv) && database)
         {
-            DIR_Server *server = (DIR_Server *)pDirectories->ElementAt(i);
-            if (server->dirType == PABDirectory)
-            {
-                /* check: this is a 4.x file, remove when conversion is done */
-                PRUint32 fileNameLen = strlen(server->fileName);
-                if ((fileNameLen > kABFileName_PreviousSuffixLen) && 
-                     strcmp(server->fileName + fileNameLen - kABFileName_PreviousSuffixLen, kABFileName_PreviousSuffix) == 0)
-                    continue;
-
-                nsCOMPtr<nsIAddrDatabase> database;
-                nsresult rv = GetAbDatabaseFromFile(server->fileName, getter_AddRefs(database));                
-                if (NS_SUCCEEDED(rv) && database)
-                {
-                    database->FindMailListbyUnicodeName(name, exist);
-                    if (*exist == PR_TRUE)
-                        return NS_OK;
-                }
-            }
+          database->FindMailListbyUnicodeName(name, exist);
+          if (*exist)
+            return NS_OK;
         }
+      }
     }
-    return NS_OK;
+  }
+  return NS_OK;
 }
 
 class AddressBookParser 
@@ -389,12 +345,15 @@ protected:
     PRBool mStoreLocAsHome;
     PRBool mDeleteDB;
     PRBool mImportingComm4x;
+    PRInt32 mLFCount;
+    PRInt32 mCRCount;
 
     nsresult ParseLDIFFile();
     void AddLdifRowToDatabase(PRBool bIsList);
     void AddLdifColToDatabase(nsIMdbRow* newRow, char* typeSlot, char* valueSlot, PRBool bIsList);
+    void ClearLdifRecordBuffer();
 
-    nsresult GetLdifStringRecord(char* buf, PRInt32 len, PRInt32* stopPos);
+    nsresult GetLdifStringRecord(char* buf, PRInt32 len, PRInt32& stopPos);
     nsresult str_parse_line(char *line, char    **type, char **value, int *vlen);
     char * str_getline( char **next );
 
@@ -417,6 +376,8 @@ AddressBookParser::AddressBookParser(nsIFileSpec * fileSpec, PRBool migrating, n
     mDeleteDB = PR_TRUE;
   mStoreLocAsHome = bStoreLocAsHome;
   mImportingComm4x = bImportingComm4x;
+  mLFCount = 0;                                                              
+  mCRCount = 0;
 }
 
 AddressBookParser::~AddressBookParser(void)
@@ -441,12 +402,11 @@ nsresult AddressBookParser::ParseFile()
     return ParseLDIFFile();
   }
  
+  // We are migrating 4.x profile
     /* Get database file name */
     char *leafName = nsnull;
-    nsAutoString fileString;
     if (mFileSpec) {
         mFileSpec->GetLeafName(&leafName);
-        fileString.AssignWithConversion(leafName);
 
         PRInt32 i = 0;
         while (leafName[i] != '\0')
@@ -490,27 +450,31 @@ nsresult AddressBookParser::ParseFile()
     nsCOMPtr<nsIRDFService> rdfService = do_GetService (NS_RDF_CONTRACTID "/rdf-service;1", &rv);
     NS_ENSURE_SUCCESS(rv, rv);
     nsCOMPtr<nsIRDFResource> parentResource;
-    char *parentUri = PR_smprintf("%s", kAllDirectoryRoot);
-    rv = rdfService->GetResource(parentUri, getter_AddRefs(parentResource));
+    rv = rdfService->GetResource(NS_LITERAL_CSTRING(kAllDirectoryRoot),
+                                 getter_AddRefs(parentResource));
     nsCOMPtr<nsIAbDirectory> parentDir = do_QueryInterface(parentResource);
     if (!parentDir)
         return NS_ERROR_NULL_POINTER;
-    if (parentUri)
-        PR_smprintf_free(parentUri);
 
-    if (PL_strcmp(fileName, kPersonalAddressbook) == 0)
-    {
-        // This is the personal address book, get name from prefs
-        nsCOMPtr<nsIPref> pPref(do_GetService(NS_PREF_CONTRACTID, &rv)); 
-        if (NS_FAILED(rv) || !pPref) 
-            return nsnull;
-        
-            nsXPIDLString dirName;
+    // Get Pretty name from prefs.
+    nsCOMPtr<nsIPref> pPref(do_GetService(NS_PREF_CONTRACTID, &rv)); 
+    if (NS_FAILED(rv) || !pPref) 
+        return nsnull;
+
+    nsXPIDLString dirName;
+    if (strcmp(fileName, kPersonalAddressbook) == 0)
         rv = pPref->GetLocalizedUnicharPref("ldap_2.servers.pab.description", getter_Copies(dirName));
-        parentDir->CreateDirectoryByURI(dirName, mDbUri, mMigrating);
-    }
     else
-        parentDir->CreateDirectoryByURI(fileString.get(), mDbUri, mMigrating);
+    {
+      nsCAutoString prefName;
+      prefName = NS_LITERAL_CSTRING("ldap_2.servers.") + nsDependentCString(leafName) + NS_LITERAL_CSTRING(".description");
+      rv = pPref->GetLocalizedUnicharPref(prefName.get(), getter_Copies(dirName));
+    }
+
+    // If a name is found then use it, otherwise use the filename as last resort.
+    if (NS_FAILED(rv) || dirName.IsEmpty())
+      dirName = NS_ConvertASCIItoUCS2(leafName);
+    parentDir->CreateDirectoryByURI(dirName, mDbUri, mMigrating);
         
     rv = ParseLDIFFile();
 
@@ -695,43 +659,41 @@ char * AddressBookParser::str_getline( char **next )
  * get one ldif record
  * 
  */
-nsresult AddressBookParser::GetLdifStringRecord(char* buf, PRInt32 len, PRInt32* stopPos)
+nsresult AddressBookParser::GetLdifStringRecord(char* buf, PRInt32 len, PRInt32& stopPos)
 {
-    PRInt32 LFCount = 0;
-    PRInt32 CRCount = 0;
-
-    for (; *stopPos < len; (*stopPos)++) 
+    for (; stopPos < len; stopPos++) 
     {
-        char c = buf[*stopPos];
+        char c = buf[stopPos];
 
         if (c == 0xA)
         {
-            LFCount++;
+            mLFCount++;
         }
         else if (c == 0xD)
         {
-            CRCount++;
+            mCRCount++;
         }
         else if ( c != 0xA && c != 0xD)
         {
-            if (LFCount == 0 && CRCount == 0)
+            if (mLFCount == 0 && mCRCount == 0)
                  mLine.Append(c);
-            else if (( LFCount > 1) || ( CRCount > 2 && LFCount ) ||
-                ( !LFCount && CRCount > 1 ))
+            else if (( mLFCount > 1) || ( mCRCount > 2 && mLFCount ) ||
+                ( !mLFCount && mCRCount > 1 ))
             {
                 return NS_OK;
             }
-            else if ((LFCount == 1 || CRCount == 1))
+            else if ((mLFCount == 1 || mCRCount == 1))
             {
                  mLine.Append('\n');
                  mLine.Append(c);
-                LFCount = 0;
-                CRCount = 0;
+                mLFCount = 0;
+                mCRCount = 0;
             }
         }
     }
-    if (*stopPos ==len && (LFCount > 1) || (CRCount > 2 && LFCount) ||
-        (!LFCount && CRCount > 1))
+
+    if ((stopPos == len) && (mLFCount > 1) || (mCRCount > 2 && mLFCount) ||
+        (!mLFCount && mCRCount > 1))
         return NS_OK;
     else
         return NS_ERROR_FAILURE;
@@ -744,7 +706,8 @@ nsresult AddressBookParser::ParseLDIFFile()
     PRInt32 startPos = 0;
     PRInt32 len = 0;
     PRBool bEof = PR_FALSE;
-    nsVoidArray listPosArray;
+    nsVoidArray listPosArray;   // where each list/group starts in ldif file
+    nsVoidArray listSizeArray;  // size of the list/group info
     PRInt32 savedStartPos = 0;
     PRInt32 filePos = 0;
 
@@ -754,7 +717,7 @@ nsresult AddressBookParser::ParseLDIFFile()
         {
             startPos = 0;
 
-            while (NS_SUCCEEDED(GetLdifStringRecord(buf, len, &startPos)))
+            while (NS_SUCCEEDED(GetLdifStringRecord(buf, len, startPos)))
             {
                 if (mLine.Find("groupOfNames") == kNotFound)
                     AddLdifRowToDatabase(PR_FALSE);
@@ -762,8 +725,8 @@ nsresult AddressBookParser::ParseLDIFFile()
                 {
                     //keep file position for mailing list
                     listPosArray.AppendElement((void*)savedStartPos);
-                    if (mLine.Length() > 0)
-                        mLine.Truncate();
+                    listSizeArray.AppendElement((void*)(filePos + startPos-savedStartPos));
+                    ClearLdifRecordBuffer();
                 }
                 savedStartPos = filePos + startPos;
             }
@@ -775,18 +738,26 @@ nsresult AddressBookParser::ParseLDIFFile()
         AddLdifRowToDatabase(PR_FALSE); 
 
     // mail Lists
-    PRInt32 i;
+    PRInt32 i, pos, size;
     PRInt32 listTotal = listPosArray.Count();
+    char *listBuf;
+    ClearLdifRecordBuffer();  // make sure the buffer is clean
     for (i = 0; i < listTotal; i++)
     {
-        PRInt32 pos = NS_PTR_TO_INT32(listPosArray.ElementAt(i));
+        pos  = NS_PTR_TO_INT32(listPosArray.ElementAt(i));
+        size = NS_PTR_TO_INT32(listSizeArray.ElementAt(i));
         if (NS_SUCCEEDED(mFileSpec->Seek(pos)))
         {
-            if (NS_SUCCEEDED(mFileSpec->Read(&pBuf, (PRInt32)sizeof(buf), &len)) && len > 0)
+
+            // Allocate enough space for the lists/groups as the size varies.
+            listBuf = (char *) PR_Malloc(size);
+            if (!listBuf)
+              continue;
+            if (NS_SUCCEEDED(mFileSpec->Read(&listBuf, size, &len)) && len > 0)
             {
                 startPos = 0;
 
-                while (NS_SUCCEEDED(GetLdifStringRecord(buf, len, &startPos)))
+                while (NS_SUCCEEDED(GetLdifStringRecord(listBuf, len, startPos)))
                 {
                     if (mLine.Find("groupOfNames") != kNotFound)
                     {
@@ -796,6 +767,7 @@ nsresult AddressBookParser::ParseLDIFFile()
                     }
                 }
             }
+            PR_FREEIF(listBuf);
         }
     }
     return NS_OK;
@@ -803,6 +775,14 @@ nsresult AddressBookParser::ParseLDIFFile()
 
 void AddressBookParser::AddLdifRowToDatabase(PRBool bIsList)
 {
+    // If no data to process then reset CR/LF counters and return.
+    if (mLine.IsEmpty())
+    {
+      mLFCount = 0;
+      mCRCount = 0;
+      return;
+    }
+
     nsCOMPtr <nsIMdbRow> newRow;
     if (mDatabase)
     {
@@ -832,14 +812,24 @@ void AddressBookParser::AddLdifRowToDatabase(PRBool bIsList)
         else
             continue; // parse error: continue with next loop iteration
     }
-    delete [] saveCursor;
+    nsMemory::Free(saveCursor);
     mDatabase->AddCardRowToDB(newRow);    
 
     if (bIsList)
         mDatabase->AddListDirNode(newRow);
 
-    if (mLine.Length() > 0)
-        mLine.Truncate();
+    // Clear buffer for next record
+    ClearLdifRecordBuffer();
+}
+
+void AddressBookParser::ClearLdifRecordBuffer()
+{
+  if (mLine.Length() > 0)
+  {
+      mLine.Truncate();
+      mLFCount = 0;
+      mCRCount = 0;
+  }
 }
 
 // We have two copies of this function in the code, one here for import and 
@@ -1248,7 +1238,11 @@ NS_IMETHODIMP nsAddressBook::ConvertLDIFtoMAB(nsIFileSpec *fileSpec, PRBool migr
     rv = abParser.ParseFile();
     NS_ENSURE_SUCCESS(rv, rv);
 
-    rv = fileSpec->CloseStream();
+    fileSpec->CloseStream();
+
+    // Commit the changes if 'db' is set.
+    if (db)
+      rv = db->Commit(nsAddrDBCommitType::kLargeCommit);
     return rv;
 }
 
@@ -1263,7 +1257,14 @@ NS_IMETHODIMP nsAddressBook::ConvertLDIFtoMAB(nsIFileSpec *fileSpec, PRBool migr
 #define TAB_FILE_EXTENSION ".tab"
 #define TXT_FILE_EXTENSION ".txt"
 #define LDIF_FILE_EXTENSION ".ldi"
-#define EXTENSION_LENGTH 4
+#define LDIF_FILE_EXTENSION2 ".ldif"
+
+enum ADDRESSBOOK_EXPORT_FILE_TYPE 
+{
+ LDIF_EXPORT_TYPE =  0,
+ CSV_EXPORT_TYPE = 1,
+ TAB_EXPORT_TYPE = 2
+};
 
 NS_IMETHODIMP nsAddressBook::ExportAddressBook(nsIAbDirectory *aDirectory)
 {
@@ -1289,7 +1290,7 @@ NS_IMETHODIMP nsAddressBook::ExportAddressBook(nsIAbDirectory *aDirectory)
   rv = bundle->GetStringFromName(NS_LITERAL_STRING("LDIFFiles").get(), getter_Copies(filterString));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = filePicker->AppendFilter(filterString.get(), NS_LITERAL_STRING("*.ldi;*.ldif").get());
+  rv = filePicker->AppendFilter(filterString.get(), NS_LITERAL_STRING("*.ldi; *.ldif").get());
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = bundle->GetStringFromName(NS_LITERAL_STRING("CSVFiles").get(), getter_Copies(filterString));
@@ -1301,7 +1302,7 @@ NS_IMETHODIMP nsAddressBook::ExportAddressBook(nsIAbDirectory *aDirectory)
   rv = bundle->GetStringFromName(NS_LITERAL_STRING("TABFiles").get(), getter_Copies(filterString));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = filePicker->AppendFilter(filterString.get(), NS_LITERAL_STRING("*.tab;*.txt").get());
+  rv = filePicker->AppendFilter(filterString.get(), NS_LITERAL_STRING("*.tab; *.txt").get());
   NS_ENSURE_SUCCESS(rv, rv);
 
   PRInt16 dialogResult;
@@ -1324,36 +1325,55 @@ NS_IMETHODIMP nsAddressBook::ExportAddressBook(nsIAbDirectory *aDirectory)
     }
   }
 
-  nsCAutoString leafName;
-  rv = localFile->GetNativeLeafName(leafName);
+  // The type of export is determined by the drop-down in
+  // the file picker dialog.
+  PRInt32 exportType;
+  rv = filePicker->GetFilterIndex(&exportType);
   NS_ENSURE_SUCCESS(rv,rv);
 
-  // we're looking for .tab, .txt, .csv, .ldi or .ldif
-  // since .ldif is the desired default, we can do this:
-  // get the last four characters of the filename.
-  // check if they equal .tab,.txt, or .csv, if so, use those
-  // otherwise, treat as LDIF
-  if (leafName.Length() > EXTENSION_LENGTH) {
-    nsCAutoString extension(Substring(leafName, leafName.Length() - EXTENSION_LENGTH, EXTENSION_LENGTH));
-
-    // treat .TXT, .Tab, .csV like .txt, .tab, and .csv
-    ToLowerCase(extension);
+  nsAutoString fileName;
+  rv = localFile->GetLeafName(fileName);
+  NS_ENSURE_SUCCESS(rv, rv);
     
-    if (extension.Equals(CSV_FILE_EXTENSION))
-      rv = ExportDirectoryToDelimitedText(aDirectory, CSV_DELIM, CSV_DELIM_LEN, localFile);
-    else if (extension.Equals(TAB_FILE_EXTENSION) || extension.Equals(TXT_FILE_EXTENSION)) {
-      // .tab and .txt are both tab seperated
-      rv = ExportDirectoryToDelimitedText(aDirectory, TAB_DELIM, TAB_DELIM_LEN, localFile);
+  switch ( exportType )
+  {
+    default:
+    case LDIF_EXPORT_TYPE: // ldif
+      // If filename does not have the correct ext, add one.
+      if ((fileName.RFind(LDIF_FILE_EXTENSION, PR_TRUE, -1, sizeof(LDIF_FILE_EXTENSION)-1) == kNotFound) &&
+       (fileName.RFind(LDIF_FILE_EXTENSION2, PR_TRUE, -1, sizeof(LDIF_FILE_EXTENSION2)-1) == kNotFound)) {
+
+       // Add the extenstion and build a new localFile.
+       fileName.Append(NS_LITERAL_STRING(LDIF_FILE_EXTENSION2));
+       localFile->SetLeafName(fileName);
     }
-    else {
-      // it's ".ldi", "ldif", or something else.  we default to LDIF
       rv = ExportDirectoryToLDIF(aDirectory, localFile);
+      break;
+
+    case CSV_EXPORT_TYPE: // csv
+      // If filename does not have the correct ext, add one.
+      if (fileName.RFind(CSV_FILE_EXTENSION, PR_TRUE, -1, sizeof(CSV_FILE_EXTENSION)-1) == kNotFound) {
+
+       // Add the extenstion and build a new localFile.
+       fileName.Append(NS_LITERAL_STRING(CSV_FILE_EXTENSION));
+       localFile->SetLeafName(fileName);
     }
+      rv = ExportDirectoryToDelimitedText(aDirectory, CSV_DELIM, CSV_DELIM_LEN, localFile);
+      break;
+
+    case TAB_EXPORT_TYPE: // tab & text
+      // If filename does not have the correct ext, add one.
+      if ( (fileName.RFind(TXT_FILE_EXTENSION, PR_TRUE, -1, sizeof(TXT_FILE_EXTENSION)-1) == kNotFound) &&
+          (fileName.RFind(TAB_FILE_EXTENSION, PR_TRUE, -1, sizeof(TAB_FILE_EXTENSION)-1) == kNotFound) ) {
+
+       // Add the extenstion and build a new localFile.
+       fileName.Append(NS_LITERAL_STRING(TXT_FILE_EXTENSION));
+       localFile->SetLeafName(fileName);
   }
-  else {
-    // the file name is too short for a proper extension, so default to LDIF
-    rv = ExportDirectoryToLDIF(aDirectory, localFile);
-  }
+      rv = ExportDirectoryToDelimitedText(aDirectory, TAB_DELIM, TAB_DELIM_LEN, localFile);
+      break;
+  };
+ 
   return rv;
 }
 
@@ -1440,7 +1460,29 @@ nsAddressBook::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory, const
             if (EXPORT_ATTRIBUTES_TABLE[i].includeForPlainText) {
               rv = card->GetCardValue(EXPORT_ATTRIBUTES_TABLE[i].abColName, getter_Copies(value));
               NS_ENSURE_SUCCESS(rv,rv);
-             
+
+              // If a string contains at least one comma, tab or double quote then
+              // we need to quote the entire string. Also if double quote is part
+              // of the string we need to quote the double quote(s) as well.  
+              nsAutoString newValue(value);
+              PRBool needsQuotes = PR_FALSE;
+              if(newValue.FindChar('"') != kNotFound)
+              {
+                needsQuotes = PR_TRUE;
+                newValue.ReplaceSubstring(NS_LITERAL_STRING("\"").get(), NS_LITERAL_STRING("\"\"").get());
+              }
+              if (!needsQuotes && (newValue.FindChar(',') != kNotFound || newValue.FindChar('\x09') != kNotFound))
+                needsQuotes = PR_TRUE;
+
+              if (needsQuotes)
+              {
+                newValue.Insert(NS_LITERAL_STRING("\""), 0);
+                newValue.Append(NS_LITERAL_STRING("\""));
+              }
+
+              // For notes, make sure CR/LF is converted to spaces 
+              // to avoid creating multiple lines for a single card.
+              //
               // the import code expects .txt, .tab, .csv files to
               // have non-ASCII data in the system charset
               //
@@ -1450,7 +1492,16 @@ nsAddressBook::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory, const
               //
               // the solution is to export / import as LDIF.
               // non-ASCII data is treated as base64 encoded UTF-8 in LDIF
-              rv = importService->SystemStringFromUnicode(value.get(), valueCStr);
+              if (!strcmp(EXPORT_ATTRIBUTES_TABLE[i].abColName, kNotesColumn))
+              {
+                if (!newValue.IsEmpty())
+                {
+                  newValue.ReplaceChar(nsCRT::CR, ' ');
+                  newValue.ReplaceChar(nsCRT::LF, ' ');
+                }
+              }
+              rv = importService->SystemStringFromUnicode(newValue.get(), valueCStr);
+
               if (NS_FAILED(rv)) {
                 NS_ASSERTION(0, "failed to convert string to system charset.  use LDIF");
                 valueCStr = "?";
@@ -1468,6 +1519,7 @@ nsAddressBook::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory, const
             else {
               // something we don't support for the current export
               // for example, .tab doesn't export preferred html format
+              continue; // go to next field
             }
 
             if (i < EXPORT_ATTRIBUTES_TABLE_COUNT - 1) {
@@ -1478,7 +1530,7 @@ nsAddressBook::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory, const
             }
           }
 
-          // write out the linebreak that seperates the cards
+          // write out the linebreak that separates the cards
           rv = outputStream->Write(MSG_LINEBREAK, MSG_LINEBREAK_LEN, &writeCount);
           NS_ENSURE_SUCCESS(rv,rv);
           if (MSG_LINEBREAK_LEN != writeCount)
@@ -1595,8 +1647,8 @@ nsAddressBook::ExportDirectoryToLDIF(nsIAbDirectory *aDirectory, nsILocalFile *a
               // ldif doesn't export mutliple addresses
             }
           }
-
-          // write out the linebreak that seperates the cards
+        
+          // write out the linebreak that separates the cards
           rv = outputStream->Write(LDIF_LINEBREAK, LDIF_LINEBREAK_LEN, &writeCount);
           NS_ENSURE_SUCCESS(rv,rv);
           if (LDIF_LINEBREAK_LEN != writeCount)
@@ -1614,7 +1666,7 @@ nsAddressBook::ExportDirectoryToLDIF(nsIAbDirectory *aDirectory, nsILocalFile *a
   return NS_OK;
 }
 
-nsresult nsAddressBook::AppendLDIFForMailList(nsIAbCard *aCard, nsAFlatCString &aResult)
+nsresult nsAddressBook::AppendLDIFForMailList(nsIAbCard *aCard, nsACString &aResult)
 {
   nsresult rv;
   nsXPIDLString attrValue;
@@ -1659,7 +1711,7 @@ nsresult nsAddressBook::AppendLDIFForMailList(nsIAbCard *aCard, nsAFlatCString &
   NS_ENSURE_SUCCESS(rv,rv);
   
   nsCOMPtr <nsIRDFResource> resource;
-  rv = rdfService->GetResource(mailListURI.get(), getter_AddRefs(resource));
+  rv = rdfService->GetResource(mailListURI, getter_AddRefs(resource));
   NS_ENSURE_SUCCESS(rv,rv);
     
   nsCOMPtr <nsIAbDirectory> mailList = do_QueryInterface(resource, &rv);
@@ -1673,9 +1725,7 @@ nsresult nsAddressBook::AppendLDIFForMailList(nsIAbCard *aCard, nsAFlatCString &
     if (total) {
       PRUint32 i;
       for (i = 0; i < total; i++) {
-        nsCOMPtr <nsISupports> item = getter_AddRefs(addresses->ElementAt(i));
-        
-        nsCOMPtr <nsIAbCard> listCard = do_QueryInterface(item, &rv);
+        nsCOMPtr <nsIAbCard> listCard = do_QueryElementAt(addresses, i, &rv);
         NS_ENSURE_SUCCESS(rv,rv);
 
         rv = AppendDNForCard("member", listCard, aResult);
@@ -1690,7 +1740,7 @@ nsresult nsAddressBook::AppendLDIFForMailList(nsIAbCard *aCard, nsAFlatCString &
   return NS_OK;
 }
 
-nsresult nsAddressBook::AppendDNForCard(const char *aProperty, nsIAbCard *aCard, nsAFlatCString &aResult)
+nsresult nsAddressBook::AppendDNForCard(const char *aProperty, nsIAbCard *aCard, nsACString &aResult)
 {
   nsXPIDLString email;
   nsXPIDLString displayName;
@@ -1719,7 +1769,7 @@ nsresult nsAddressBook::AppendDNForCard(const char *aProperty, nsIAbCard *aCard,
   return rv;
 }
 
-nsresult nsAddressBook::AppendBasicLDIFForCard(nsIAbCard *aCard, nsAFlatCString &aResult)
+nsresult nsAddressBook::AppendBasicLDIFForCard(nsIAbCard *aCard, nsACString &aResult)
 {
   nsresult rv = AppendDNForCard("dn", aCard, aResult);
   NS_ENSURE_SUCCESS(rv,rv);
@@ -1755,7 +1805,7 @@ PRBool nsAddressBook::IsSafeLDIFString(const PRUnichar *aStr)
   return PR_TRUE;
 }
 
-nsresult nsAddressBook::AppendProperty(const char *aProperty, const PRUnichar *aValue, nsAFlatCString &aResult)
+nsresult nsAddressBook::AppendProperty(const char *aProperty, const PRUnichar *aValue, nsACString &aResult)
 {
   NS_ENSURE_ARG_POINTER(aValue);
 
