@@ -34,7 +34,7 @@
 /*
  * Test program for SDR (Secret Decoder Ring) functions.
  *
- * $Id: sdrtest.c,v 1.6.20.1 2002/04/10 03:25:51 cltbld%netscape.com Exp $
+ * $Id: sdrtest.c,v 1.10 2003/04/01 22:03:07 wtc%netscape.com Exp $
  */
 
 #include "nspr.h"
@@ -100,7 +100,6 @@ main (int argc, char **argv)
 {
     int		 retval = 0;  /* 0 - test succeeded.  -1 - test failed */
     SECStatus	 rv;
-    const char	*certDir = ".";
     PLOptState	*optstate;
     char	*program_name;
     const char  *input_file = NULL; 	/* read encrypted data from here (or create) */
@@ -233,6 +232,8 @@ file_loser:
                                     PK11_GetSlotName(slot));
             return SECFailure;
         }
+      }
+      if (slot) {
 	PK11_FreeSlot(slot);
       }
 
@@ -294,7 +295,9 @@ file_loser:
 loser:
     if (text.data) free(text.data);
     if (result.data) free(result.data);
-    NSS_Shutdown();
+    if (NSS_Shutdown() != SECSuccess) {
+       exit(1);
+    }
 
 prdone:
     PR_Cleanup ();
