@@ -32,7 +32,7 @@
  */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: list.c,v $ $Revision: 1.16.6.1 $ $Date: 2002/04/10 03:26:29 $ $Name: MOZILLA_1_0_RELEASE $";
+static const char CVS_ID[] = "@(#) $RCSfile: list.c,v $ $Revision: 1.17 $ $Date: 2002/04/22 20:44:07 $ $Name: MOZILLA_1_4_1_RELEASE $";
 #endif /* DEBUG */
 
 /*
@@ -313,19 +313,23 @@ nssList_Count(nssList *list)
 NSS_IMPLEMENT PRStatus
 nssList_GetArray(nssList *list, void **rvArray, PRUint32 maxElements)
 {
-    nssListIterator *iter;
-    void *el;
+    nssListElement *node;
     PRUint32 i = 0;
     PR_ASSERT(maxElements > 0);
-    iter = nssList_CreateIterator(list);
-    for (el = nssListIterator_Start(iter); el != NULL;
-         el = nssListIterator_Next(iter)) 
-    {
-	rvArray[i++] = el;
-	if (i == maxElements) break;
+    node = list->head;
+    if (!node) {
+	return PR_SUCCESS;
     }
-    nssListIterator_Finish(iter);
-    nssListIterator_Destroy(iter);
+    NSSLIST_LOCK_IF(list);
+    while (node) {
+	rvArray[i++] = node->data;
+	if (i == maxElements) break;
+	node = (nssListElement *)PR_NEXT_LINK(&node->link);
+	if (node == list->head) {
+	    break;
+	}
+    }
+    NSSLIST_UNLOCK_IF(list);
     return PR_SUCCESS;
 }
 
