@@ -32,7 +32,7 @@
  */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: instance.c,v $ $Revision: 1.5.6.1 $ $Date: 2002/04/10 03:26:42 $ $Name: MOZILLA_1_0_RELEASE $";
+static const char CVS_ID[] = "@(#) $RCSfile: instance.c,v $ $Revision: 1.7 $ $Date: 2002/10/10 22:39:22 $ $Name: MOZILLA_1_4_1_RELEASE $";
 #endif /* DEBUG */
 
 /*
@@ -394,6 +394,14 @@ nssCKFWInstance_Destroy
     fwInstance->mdInstance->Finalize(fwInstance->mdInstance, fwInstance);
   }
 
+  if (fwInstance->sessionHandleHash) {
+     nssCKFWHash_Destroy(fwInstance->sessionHandleHash);
+  }
+
+  if (fwInstance->objectHandleHash) {
+     nssCKFWHash_Destroy(fwInstance->objectHandleHash);
+  }
+
 #ifdef DEBUG
   (void)instance_remove_pointer(fwInstance);
 #endif /* DEBUG */
@@ -1092,7 +1100,7 @@ nssCKFWInstance_GetLibraryVersion
       fwInstance->mdInstance, fwInstance);
   } else {
     fwInstance->libraryVersion.major = 0;
-    fwInstance->libraryVersion.minor = 1;
+    fwInstance->libraryVersion.minor = 3;
   }
 
   rv = fwInstance->libraryVersion;
