@@ -32,7 +32,7 @@
  */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: token.c,v $ $Revision: 1.7 $ $Date: 2001/11/16 19:41:49 $ $Name:  $";
+static const char CVS_ID[] = "@(#) $RCSfile: token.c,v $ $Revision: 1.8 $ $Date: 2002/10/10 22:39:23 $ $Name: MOZILLA_1_4_1_RELEASE $";
 #endif /* DEBUG */
 
 /*
@@ -307,6 +307,22 @@ nssCKFWToken_Create
   return (NSSCKFWToken *)NULL;
 }
 
+static void
+nss_ckfwtoken_session_iterator
+(
+  const void *key,
+  void *value,
+  void *closure
+)
+{
+  /*
+   * Remember that the fwToken->mutex is locked
+   */
+  NSSCKFWSession *fwSession = (NSSCKFWSession *)value;
+  (void)nssCKFWSession_Destroy(fwSession, CK_FALSE);
+  return;
+}
+
 /*
  * nssCKFWToken_Destroy
  *
@@ -332,6 +348,19 @@ nssCKFWToken_Destroy
     fwToken->mdToken->Invalidate(fwToken->mdToken, fwToken,
       fwToken->mdInstance, fwToken->fwInstance);
   }
+  /* we can destroy the list without locking now because no one else is 
+   * referencing us (or _Destroy was invalidly called!)
+   */
+  nssCKFWHash_Iterate(fwToken->sessions, nss_ckfwtoken_session_iterator, 
+								(void *)NULL);
+  nssCKFWHash_Destroy(fwToken->sessions);
+
+  if (fwToken->sessionObjectHash) {
+    nssCKFWHash_Destroy(fwToken->sessionObjectHash);
+  }
+  if (fwToken->mdObjectHash) {
+    nssCKFWHash_Destroy(fwToken->mdObjectHash);
+  }
 
   nssCKFWSlot_ClearToken(fwToken->fwSlot);
   
@@ -1547,21 +1576,6 @@ nssCKFWToken_RemoveSession
   return error;
 }
 
-static void
-nss_ckfwtoken_session_iterator
-(
-  const void *key,
-  void *value,
-  void *closure
-)
-{
-  /*
-   * Remember that the fwToken->mutex is locked
-   */
-  NSSCKFWSession *fwSession = (NSSCKFWSession *)value;
-  (void)nssCKFWSession_Destroy(fwSession, CK_FALSE);
-  return;
-}
 
 /*
  * nssCKFWToken_CloseAllSessions
