@@ -32,7 +32,7 @@
  */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: certificate.c,v $ $Revision: 1.31.4.1 $ $Date: 2002/03/08 22:18:42 $ $Name:  $";
+static const char CVS_ID[] = "@(#) $RCSfile: certificate.c,v $ $Revision: 1.46 $ $Date: 2003/01/30 05:12:10 $ $Name: MOZILLA_1_4_1_RELEASE $";
 #endif /* DEBUG */
 
 #ifndef NSSPKI_H
@@ -51,8 +51,12 @@ static const char CVS_ID[] = "@(#) $RCSfile: certificate.c,v $ $Revision: 1.31.4
 #include "dev.h"
 #endif /* DEV_H */
 
+#include "pkistore.h"
+
 #ifdef NSS_3_4_CODE
 #include "pki3hack.h"
+#include "pk11func.h"
+#include "hasht.h"
 #endif
 
 #ifndef BASE_H
@@ -61,9 +65,44 @@ static const char CVS_ID[] = "@(#) $RCSfile: certificate.c,v $ $Revision: 1.31.4
 
 extern const NSSError NSS_ERROR_NOT_FOUND;
 
+/* Creates a certificate from a base object */
 NSS_IMPLEMENT NSSCertificate *
-nssCertificate_AddRef
-(
+nssCertificate_Create (
+  nssPKIObject *object
+)
+{
+    PRStatus status;
+    NSSCertificate *rvCert;
+    /* mark? */
+    NSSArena *arena = object->arena;
+    PR_ASSERT(object->instances != NULL && object->numInstances > 0);
+    rvCert = nss_ZNEW(arena, NSSCertificate);
+    if (!rvCert) {
+	return (NSSCertificate *)NULL;
+    }
+    rvCert->object = *object;
+    /* XXX should choose instance based on some criteria */
+    status = nssCryptokiCertificate_GetAttributes(object->instances[0],
+                                                  NULL,  /* XXX sessionOpt */
+                                                  arena,
+                                                  &rvCert->type,
+                                                  &rvCert->id,
+                                                  &rvCert->encoding,
+                                                  &rvCert->issuer,
+                                                  &rvCert->serial,
+                                                  &rvCert->subject);
+    if (status != PR_SUCCESS) {
+	return (NSSCertificate *)NULL;
+    }
+    /* all certs need an encoding value */
+    if (rvCert->encoding.data == NULL) {
+	return (NSSCertificate *)NULL;
+    }
+    return rvCert;
+}
+
+NSS_IMPLEMENT NSSCertificate *
+nssCertificate_AddRef (
   NSSCertificate *c
 )
 {
@@ -74,102 +113,137 @@ nssCertificate_AddRef
 }
 
 NSS_IMPLEMENT PRStatus
-NSSCertificate_Destroy
-(
+nssCertificate_Destroy (
   NSSCertificate *c
 )
 {
-    PRBool destroyed;
     if (c) {
+	PRUint32 i;
 	nssDecodedCert *dc = c->decoding;
-	destroyed = nssPKIObject_Destroy(&c->object);
-	if (destroyed) {
-	    if (dc) {
-		nssDecodedCert_Destroy(dc);
+	NSSTrustDomain *td = STAN_GetDefaultTrustDomain();
+	NSSCryptoContext *cc = c->object.cryptoContext;
+
+	PR_ASSERT(c->object.refCount > 0);
+
+	/* --- LOCK storage --- */
+	if (cc) {
+	    nssCertificateStore_Lock(cc->certStore);
+	} else {
+	    nssTrustDomain_LockCertCache(td);
+	}
+	PR_AtomicDecrement(&c->object.refCount);
+	if (c->object.refCount == 0) {
+	    /* --- remove cert and UNLOCK storage --- */
+	    if (cc) {
+		nssCertificateStore_RemoveCertLOCKED(cc->certStore, c);
+		nssCertificateStore_Unlock(cc->certStore);
+	    } else {
+		nssTrustDomain_RemoveCertFromCacheLOCKED(td, c);
+		nssTrustDomain_UnlockCertCache(td);
+	    }
+	    /* free cert data */
+	    for (i=0; i<c->object.numInstances; i++) {
+		nssCryptokiObject_Destroy(c->object.instances[i]);
+	    }
+	    PZ_DestroyLock(c->object.lock);
+	    nssArena_Destroy(c->object.arena);
+	    nssDecodedCert_Destroy(dc);
+	} else {
+	    /* --- UNLOCK storage --- */
+	    if (cc) {
+		nssCertificateStore_Unlock(cc->certStore);
+	    } else {
+		nssTrustDomain_UnlockCertCache(td);
 	    }
 	}
     }
     return PR_SUCCESS;
 }
 
+NSS_IMPLEMENT PRStatus
+NSSCertificate_Destroy (
+  NSSCertificate *c
+)
+{
+    return nssCertificate_Destroy(c);
+}
+
+NSS_IMPLEMENT NSSDER *
+nssCertificate_GetEncoding (
+  NSSCertificate *c
+)
+{
+    if (c->encoding.size > 0 && c->encoding.data) {
+	return &c->encoding;
+    } else {
+	return (NSSDER *)NULL;
+    }
+}
+
+NSS_IMPLEMENT NSSDER *
+nssCertificate_GetIssuer (
+  NSSCertificate *c
+)
+{
+    if (c->issuer.size > 0 && c->issuer.data) {
+	return &c->issuer;
+    } else {
+	return (NSSDER *)NULL;
+    }
+}
+
+NSS_IMPLEMENT NSSDER *
+nssCertificate_GetSerialNumber (
+  NSSCertificate *c
+)
+{
+    if (c->serial.size > 0 && c->serial.data) {
+	return &c->serial;
+    } else {
+	return (NSSDER *)NULL;
+    }
+}
+
+NSS_IMPLEMENT NSSDER *
+nssCertificate_GetSubject (
+  NSSCertificate *c
+)
+{
+    if (c->subject.size > 0 && c->subject.data) {
+	return &c->subject;
+    } else {
+	return (NSSDER *)NULL;
+    }
+}
+
 NSS_IMPLEMENT NSSUTF8 *
-NSSCertificate_GetNickname
-(
+nssCertificate_GetNickname (
   NSSCertificate *c,
   NSSToken *tokenOpt
 )
 {
-    NSSUTF8 *rvNick = NULL;
-    nssCryptokiInstance *instance;
-    nssListIterator *instances = c->object.instances;
-    if (c->object.cryptoContext) {
-	return c->object.tempName;
-    }
-    for (instance  = (nssCryptokiInstance *)nssListIterator_Start(instances);
-         instance != (nssCryptokiInstance *)NULL;
-         instance  = (nssCryptokiInstance *)nssListIterator_Next(instances)) 
-    {
-	if (tokenOpt) {
-	    if (instance->token == tokenOpt) {
-		/* take the nickname on the given token */
-		rvNick = instance->label;
-		break;
-	    }
-	} else {
-	    /* take the first one */
-	    rvNick = instance->label;
-	    break;
-	}
-    }
-    nssListIterator_Finish(instances);
-    return rvNick;
+    return nssPKIObject_GetNicknameForToken(&c->object, tokenOpt);
+}
+
+NSS_IMPLEMENT NSSASCII7 *
+nssCertificate_GetEmailAddress (
+  NSSCertificate *c
+)
+{
+    return c->email;
 }
 
 NSS_IMPLEMENT PRStatus
-NSSCertificate_DeleteStoredObject
-(
+NSSCertificate_DeleteStoredObject (
   NSSCertificate *c,
   NSSCallback *uhh
 )
 {
-    /* this needs more thought on what will happen when there are multiple
-     * instances
-     */
-    /* XXX use callback to log in if neccessary */
-    PRStatus nssrv = PR_SUCCESS;
-    nssCryptokiInstance *instance;
-    nssListIterator *instances = c->object.instances;
-    for (instance  = (nssCryptokiInstance *)nssListIterator_Start(instances);
-         instance != (nssCryptokiInstance *)NULL;
-         instance  = (nssCryptokiInstance *)nssListIterator_Next(instances)) 
-    {
-	/* XXX this should be fixed to understand read-only tokens, for
-	 * now, to handle the builtins, just make the attempt.
-	 */
-	nssrv = nssToken_DeleteStoredObject(instance);
-	if (nssrv == PR_SUCCESS) {
-	    nssList_Remove(c->object.instanceList, instance);
-#ifdef NSS_3_4_CODE
-	    if (instance->token->certList) {
-		/* If the cert has been cached locally on the token, remove
-		 * that reference
-		 */
-		nssList_Remove(instance->token->certList, c);
-		NSSCertificate_Destroy(c);
-	    }
-#endif
-	}
-    }
-    nssListIterator_Finish(instances);
-    c->object.instances = nssList_CreateIterator(c->object.instanceList);
-    nssListIterator_Destroy(instances);
-    /* XXX for now, always success */
-    return PR_SUCCESS;
+    return nssPKIObject_DeleteStoredObject(&c->object, uhh, PR_TRUE);
 }
 
 NSS_IMPLEMENT PRStatus
-NSSCertificate_Validate
-(
+NSSCertificate_Validate (
   NSSCertificate *c,
   NSSTime *timeOpt, /* NULL for "now" */
   NSSUsage *usage,
@@ -181,8 +255,7 @@ NSSCertificate_Validate
 }
 
 NSS_IMPLEMENT void ** /* void *[] */
-NSSCertificate_ValidateCompletely
-(
+NSSCertificate_ValidateCompletely (
   NSSCertificate *c,
   NSSTime *timeOpt, /* NULL for "now" */
   NSSUsage *usage,
@@ -197,8 +270,7 @@ NSSCertificate_ValidateCompletely
 }
 
 NSS_IMPLEMENT PRStatus
-NSSCertificate_ValidateAndDiscoverUsagesAndPolicies
-(
+NSSCertificate_ValidateAndDiscoverUsagesAndPolicies (
   NSSCertificate *c,
   NSSTime **notBeforeOutOpt,
   NSSTime **notAfterOutOpt,
@@ -215,8 +287,7 @@ NSSCertificate_ValidateAndDiscoverUsagesAndPolicies
 }
 
 NSS_IMPLEMENT NSSDER *
-NSSCertificate_Encode
-(
+NSSCertificate_Encode (
   NSSCertificate *c,
   NSSDER *rvOpt,
   NSSArena *arenaOpt
@@ -227,8 +298,7 @@ NSSCertificate_Encode
 }
 
 NSS_IMPLEMENT nssDecodedCert *
-nssCertificate_GetDecoding
-(
+nssCertificate_GetDecoding (
   NSSCertificate *c
 )
 {
@@ -238,79 +308,139 @@ nssCertificate_GetDecoding
     return c->decoding;
 }
 
-static void
-nssCertificateArray_Destroy
-(
-  NSSCertificate **certArray
+static NSSCertificate **
+filter_subject_certs_for_id (
+  NSSCertificate **subjectCerts, 
+  void *id
 )
-{
-    NSSCertificate **ci;
-    ci = certArray;
-    while (ci && *ci) {
-	NSSCertificate_Destroy(*ci);
-	ci++;
-    }
-    nss_ZFreeIf(certArray);
-}
-
-static NSSCertificate *
-filter_subject_certs_for_id(NSSCertificate **subjectCerts, NSSItem *id)
 {
     NSSCertificate **si;
-    NSSCertificate *rvCert = NULL;
     nssDecodedCert *dcp;
+    int nextOpenSlot = 0;
+    int i;
+    nssCertIDMatch matchLevel = nssCertIDMatch_Unknown;
+    nssCertIDMatch match;
+
     /* walk the subject certs */
-    si = subjectCerts;
-    while (*si) {
+    for (si = subjectCerts; *si; si++) {
 	dcp = nssCertificate_GetDecoding(*si);
-	if (dcp->matchIdentifier(dcp, id)) {
-	    /* this cert has the correct identifier */
-	    rvCert = nssCertificate_AddRef(*si);
+	if (!dcp) {
+	    NSSCertificate_Destroy(*si);
+	    continue;
+	}
+	match = dcp->matchIdentifier(dcp, id);
+	switch (match) {
+	case nssCertIDMatch_Yes:
+	    if (matchLevel == nssCertIDMatch_Unknown) {
+		/* we have non-definitive matches, forget them */
+		for (i = 0; i < nextOpenSlot; i++) {
+		    NSSCertificate_Destroy(subjectCerts[i]);
+		    subjectCerts[i] = NULL;
+		}
+		nextOpenSlot = 0;
+		/* only keep definitive matches from now on */
+		matchLevel = nssCertIDMatch_Yes;
+	    }
+	    /* keep the cert */
+	    subjectCerts[nextOpenSlot++] = *si;
 	    break;
+	case nssCertIDMatch_Unknown:
+	    if (matchLevel == nssCertIDMatch_Unknown) {
+		/* only have non-definitive matches so far, keep it */
+		subjectCerts[nextOpenSlot++] = *si;
+		break;
+	    }
+	    /* else fall through, we have a definitive match already */
+	case nssCertIDMatch_No:
+	default:
+	    NSSCertificate_Destroy(*si);
+	    *si = NULL;
 	}
-	si++;
     }
-    return rvCert;
+    subjectCerts[nextOpenSlot] = NULL;
+    return subjectCerts;
+}
+
+static NSSCertificate **
+filter_certs_for_valid_issuers (
+  NSSCertificate **certs
+)
+{
+    NSSCertificate **cp;
+    nssDecodedCert *dcp;
+    int nextOpenSlot = 0;
+    int i;
+
+    for (cp = certs; *cp; cp++) {
+	dcp = nssCertificate_GetDecoding(*cp);
+	if (dcp && dcp->isValidIssuer(dcp)) {
+	    certs[nextOpenSlot++] = *cp;
+	} else {
+	    NSSCertificate_Destroy(*cp);
+	}
+    }
+    certs[nextOpenSlot] = NULL;
+    return certs;
 }
 
 static NSSCertificate *
-find_issuer_cert_for_identifier(NSSCertificate *c, NSSItem *id)
+find_cert_issuer (
+  NSSCertificate *c,
+  NSSTime *timeOpt,
+  NSSUsage *usage,
+  NSSPolicies *policiesOpt
+)
 {
-    NSSCertificate *rvCert = NULL;
-    NSSCertificate **subjectCerts = NULL;
+    NSSArena *arena;
+    NSSCertificate **certs = NULL;
+    NSSCertificate **ccIssuers = NULL;
+    NSSCertificate **tdIssuers = NULL;
+    NSSCertificate *issuer = NULL;
     NSSTrustDomain *td;
     NSSCryptoContext *cc;
-    /* Find all certs with this cert's issuer as the subject */
     cc = c->object.cryptoContext; /* NSSCertificate_GetCryptoContext(c); */
+    td = NSSCertificate_GetTrustDomain(c);
+#ifdef NSS_3_4_CODE
+    if (!td) {
+	td = STAN_GetDefaultTrustDomain();
+    }
+#endif
+    arena = nssArena_Create();
+    if (!arena) {
+	return (NSSCertificate *)NULL;
+    }
     if (cc) {
-	subjectCerts = NSSCryptoContext_FindCertificatesBySubject(cc,
-	                                                          &c->issuer,
-	                                                          NULL,
-	                                                          0,
-	                                                          NULL);
-	if (subjectCerts) {
-	    rvCert = filter_subject_certs_for_id(subjectCerts, id);
-	    nssCertificateArray_Destroy(subjectCerts);
-	}
+	ccIssuers = nssCryptoContext_FindCertificatesBySubject(cc,
+	                                                       &c->issuer,
+	                                                       NULL,
+	                                                       0,
+	                                                       arena);
     }
-    if (!rvCert) {
-	/* The general behavior of NSS <3.4 seems to be that if the search
-	 * turns up empty in the temp db, fall back to the perm db,
-	 * irregardless of whether or not the cert itself is perm or temp.
-	 * This is replicated here.
-	 */
-	td = NSSCertificate_GetTrustDomain(c);
-	subjectCerts = NSSTrustDomain_FindCertificatesBySubject(td,
-	                                                        &c->issuer,
-	                                                        NULL,
-	                                                        0,
-	                                                        NULL);
-	if (subjectCerts) {
-	    rvCert = filter_subject_certs_for_id(subjectCerts, id);
-	    nssCertificateArray_Destroy(subjectCerts);
+    tdIssuers = nssTrustDomain_FindCertificatesBySubject(td,
+                                                         &c->issuer,
+                                                         NULL,
+                                                         0,
+                                                         arena);
+    certs = nssCertificateArray_Join(ccIssuers, tdIssuers);
+    if (certs) {
+	nssDecodedCert *dc = NULL;
+	void *issuerID = NULL;
+	dc = nssCertificate_GetDecoding(c);
+	if (dc) {
+	    issuerID = dc->getIssuerIdentifier(dc);
 	}
+	if (issuerID) {
+	    certs = filter_subject_certs_for_id(certs, issuerID);
+	}
+	certs = filter_certs_for_valid_issuers(certs);
+	issuer = nssCertificateArray_FindBestCertificate(certs,
+	                                                 timeOpt,
+	                                                 usage,
+	                                                 policiesOpt);
+	nssCertificateArray_Destroy(certs);
     }
-    return rvCert;
+    nssArena_Destroy(arena);
+    return issuer;
 }
 
 /* XXX review based on CERT_FindCertIssuer
@@ -318,8 +448,85 @@ find_issuer_cert_for_identifier(NSSCertificate *c, NSSItem *id)
  * if authority key id does not exist
  */
 NSS_IMPLEMENT NSSCertificate **
-NSSCertificate_BuildChain
-(
+nssCertificate_BuildChain (
+  NSSCertificate *c,
+  NSSTime *timeOpt,
+  NSSUsage *usage,
+  NSSPolicies *policiesOpt,
+  NSSCertificate **rvOpt,
+  PRUint32 rvLimit,
+  NSSArena *arenaOpt,
+  PRStatus *statusOpt
+)
+{
+    NSSCertificate **rvChain;
+#ifdef NSS_3_4_CODE
+    NSSCertificate *cp;
+    CERTCertificate *cCert;
+#endif
+    NSSUsage issuerUsage = *usage;
+    NSSTrustDomain *td;
+    nssPKIObjectCollection *collection;
+
+    td = NSSCertificate_GetTrustDomain(c);
+#ifdef NSS_3_4_CODE
+    if (!td) {
+	td = STAN_GetDefaultTrustDomain();
+    }
+    /* bump the usage up to CA level */
+    issuerUsage.nss3lookingForCA = PR_TRUE;
+#endif
+    if (statusOpt) *statusOpt = PR_SUCCESS;
+    collection = nssCertificateCollection_Create(td, NULL);
+    if (!collection) {
+	if (statusOpt) *statusOpt = PR_FAILURE;
+	return (NSSCertificate **)NULL;
+    }
+    nssPKIObjectCollection_AddObject(collection, (nssPKIObject *)c);
+    if (rvLimit == 1) {
+	goto finish;
+    }
+    /* XXX This breaks code for which NSS_3_4_CODE is not defined (pure
+     *     4.0 builds).  That won't affect the tip.  But be careful
+     *     when merging 4.0!!!
+     */
+    while (c != (NSSCertificate *)NULL) {
+#ifdef NSS_3_4_CODE
+	cCert = STAN_GetCERTCertificate(c);
+	if (cCert->isRoot) {
+	    /* not including the issuer of the self-signed cert, which is,
+	     * of course, itself
+	     */
+	    break;
+	}
+	cp = c;
+#endif
+	c = find_cert_issuer(c, timeOpt, &issuerUsage, policiesOpt);
+	if (c) {
+	    nssPKIObjectCollection_AddObject(collection, (nssPKIObject *)c);
+	    nssCertificate_Destroy(c); /* collection has it */
+	    if (rvLimit > 0 &&
+	        nssPKIObjectCollection_Count(collection) == rvLimit) 
+	    {
+		break;
+	    }
+	} else {
+	    nss_SetError(NSS_ERROR_CERTIFICATE_ISSUER_NOT_FOUND);
+	    if (statusOpt) *statusOpt = PR_FAILURE;
+	    break;
+	}
+    }
+finish:
+    rvChain = nssPKIObjectCollection_GetCertificates(collection, 
+                                                     rvOpt, 
+                                                     rvLimit, 
+                                                     arenaOpt);
+    nssPKIObjectCollection_Destroy(collection);
+    return rvChain;
+}
+
+NSS_IMPLEMENT NSSCertificate **
+NSSCertificate_BuildChain (
   NSSCertificate *c,
   NSSTime *timeOpt,
   NSSUsage *usage,
@@ -330,143 +537,36 @@ NSSCertificate_BuildChain
   PRStatus *statusOpt
 )
 {
-    PRStatus nssrv;
-    nssList *chain;
-    NSSItem *issuerID;
-    NSSCertificate **rvChain;
-    NSSTrustDomain *td;
-    NSSCryptoContext *cc;
-    nssDecodedCert *dc;
-    NSSCertificate *ct, *cp;
-    cc = c->object.cryptoContext; /* NSSCertificate_GetCryptoContext(c); */
-    td = NSSCertificate_GetTrustDomain(c);
-#ifdef NSS_3_4_CODE
-    if (!td) {
-	td = STAN_GetDefaultTrustDomain();
-    }
-#endif
-    chain = nssList_Create(NULL, PR_FALSE);
-    nssList_Add(chain, nssCertificate_AddRef(c));
-    if (statusOpt) *statusOpt = PR_SUCCESS;
-    if (rvLimit == 1) goto finish;
-    while (!nssItem_Equal(&c->subject, &c->issuer, &nssrv)) {
-	dc = nssCertificate_GetDecoding(c);
-	issuerID = dc->getIssuerIdentifier(dc);
-	if (issuerID) {
-	    c = find_issuer_cert_for_identifier(c, issuerID);
-	    nssItem_Destroy(issuerID);
-	    issuerID = NULL;
-	    if (!c) {
-		nss_SetError(NSS_ERROR_CERTIFICATE_ISSUER_NOT_FOUND);
-		if (statusOpt) *statusOpt = PR_FAILURE;
-		goto finish;
-	    }
-	} else {
-	    nssBestCertificateCB best;
-	    NSSDER *issuer = &c->issuer;
-#ifdef NSS_3_4_CODE
-	    PRBool tmpca = usage->nss3lookingForCA;
-	    usage->nss3lookingForCA = PR_TRUE;
-#endif
-	    c = ct = cp = NULL;
-	    if (cc) {
-		ct = NSSCryptoContext_FindBestCertificateBySubject(cc,
-		                                                   issuer,
-		                                                   timeOpt,
-		                                                   usage,
-		                                                   policiesOpt);
-		/* Mimic functionality from CERT_FindCertIssuer.  If a matching
-		 * cert (based on trust & usage) cannot be found, just take the
-		 * newest cert with the correct subject.
-		 */
-		if (!ct && !usage->anyUsage) {
-		    usage->anyUsage = PR_TRUE;
-		    ct = NSSCryptoContext_FindBestCertificateBySubject(cc,
-		                                                  issuer,
-		                                                  timeOpt,
-		                                                  usage,
-		                                                  policiesOpt);
-		    usage->anyUsage = PR_FALSE;
-		}
-	    }
-	    cp = NSSTrustDomain_FindBestCertificateBySubject(td,
-	                                                     issuer,
-	                                                     timeOpt,
-	                                                     usage,
-	                                                     policiesOpt);
-	    /* Mimic functionality from CERT_FindCertIssuer.  If a matching
-	     * cert (based on trust & usage) cannot be found, just take the
-	     * newest cert with the correct subject.
-	     */
-	    if (!cp && !usage->anyUsage) {
-		usage->anyUsage = PR_TRUE;
-		cp = NSSTrustDomain_FindBestCertificateBySubject(td,
-		                                                 issuer,
-		                                                 timeOpt,
-		                                                 usage,
-		                                                 policiesOpt);
-		usage->anyUsage = PR_FALSE;
-	    }
-	    nssBestCertificate_SetArgs(&best, timeOpt, usage, policiesOpt);
-	    /* Take the better of the best temp and best perm cert, according
-	     * to the given usage
-	     */
-	    if (ct) {
-		nssBestCertificate_Callback(ct, (void *)&best);
-	    }
-	    if (cp) {
-		nssBestCertificate_Callback(cp, (void *)&best);
-	    }
-	    if (!best.cert) {
-		best.usage->anyUsage = PR_TRUE;
-		/* Take the newest of the best temp and best perm cert */
-		if (ct) {
-		    nssBestCertificate_Callback(ct, (void *)&best);
-		}
-		if (cp) {
-		    nssBestCertificate_Callback(cp, (void *)&best);
-		}
-	    }
-	    if (ct) NSSCertificate_Destroy(ct);
-	    if (cp) NSSCertificate_Destroy(cp);
-	    c = best.cert;
-#ifdef NSS_3_4_CODE
-	    usage->nss3lookingForCA = tmpca;
-#endif
-	    if (!c) {
-		nss_SetError(NSS_ERROR_CERTIFICATE_ISSUER_NOT_FOUND);
-		if (statusOpt) *statusOpt = PR_FAILURE;
-		goto finish;
-	    }
-	}
-	nssList_Add(chain, c);
-	if (nssList_Count(chain) == rvLimit) goto finish;
-    }
-finish:
-    if (rvOpt) {
-	rvChain = rvOpt;
-    } else {
-	rvLimit = nssList_Count(chain);
-	rvChain = nss_ZNEWARRAY(arenaOpt, NSSCertificate *, rvLimit + 1);
-    }
-    nssList_GetArray(chain, (void **)rvChain, rvLimit);
-    nssList_Destroy(chain);
-    /* XXX now, the question is, cache all certs in the chain? */
-    return rvChain;
+    return nssCertificate_BuildChain(c, timeOpt, usage, policiesOpt,
+                                     rvOpt, rvLimit, arenaOpt, statusOpt);
+}
+
+NSS_IMPLEMENT NSSCryptoContext *
+nssCertificate_GetCryptoContext (
+  NSSCertificate *c
+)
+{
+    return c->object.cryptoContext;
 }
 
 NSS_IMPLEMENT NSSTrustDomain *
-NSSCertificate_GetTrustDomain
-(
+nssCertificate_GetTrustDomain (
   NSSCertificate *c
 )
 {
     return c->object.trustDomain;
 }
 
+NSS_IMPLEMENT NSSTrustDomain *
+NSSCertificate_GetTrustDomain (
+  NSSCertificate *c
+)
+{
+    return nssCertificate_GetTrustDomain(c);
+}
+
 NSS_IMPLEMENT NSSToken *
-NSSCertificate_GetToken
-(
+NSSCertificate_GetToken (
   NSSCertificate *c,
   PRStatus *statusOpt
 )
@@ -475,8 +575,7 @@ NSSCertificate_GetToken
 }
 
 NSS_IMPLEMENT NSSSlot *
-NSSCertificate_GetSlot
-(
+NSSCertificate_GetSlot (
   NSSCertificate *c,
   PRStatus *statusOpt
 )
@@ -485,8 +584,7 @@ NSSCertificate_GetSlot
 }
 
 NSS_IMPLEMENT NSSModule *
-NSSCertificate_GetModule
-(
+NSSCertificate_GetModule (
   NSSCertificate *c,
   PRStatus *statusOpt
 )
@@ -495,8 +593,7 @@ NSSCertificate_GetModule
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSCertificate_Encrypt
-(
+NSSCertificate_Encrypt (
   NSSCertificate *c,
   NSSAlgorithmAndParameters *apOpt,
   NSSItem *data,
@@ -513,8 +610,7 @@ NSSCertificate_Encrypt
 }
 
 NSS_IMPLEMENT PRStatus
-NSSCertificate_Verify
-(
+NSSCertificate_Verify (
   NSSCertificate *c,
   NSSAlgorithmAndParameters *apOpt,
   NSSItem *data,
@@ -530,8 +626,7 @@ NSSCertificate_Verify
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSCertificate_VerifyRecover
-(
+NSSCertificate_VerifyRecover (
   NSSCertificate *c,
   NSSAlgorithmAndParameters *apOpt,
   NSSItem *signature,
@@ -548,8 +643,7 @@ NSSCertificate_VerifyRecover
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSCertificate_WrapSymmetricKey
-(
+NSSCertificate_WrapSymmetricKey (
   NSSCertificate *c,
   NSSAlgorithmAndParameters *apOpt,
   NSSSymmetricKey *keyToWrap,
@@ -566,8 +660,7 @@ NSSCertificate_WrapSymmetricKey
 }
 
 NSS_IMPLEMENT NSSCryptoContext *
-NSSCertificate_CreateCryptoContext
-(
+NSSCertificate_CreateCryptoContext (
   NSSCertificate *c,
   NSSAlgorithmAndParameters *apOpt,
   NSSTime *timeOpt,
@@ -581,8 +674,7 @@ NSSCertificate_CreateCryptoContext
 }
 
 NSS_IMPLEMENT NSSPublicKey *
-NSSCertificate_GetPublicKey
-(
+NSSCertificate_GetPublicKey (
   NSSCertificate *c
 )
 {
@@ -619,8 +711,7 @@ NSSCertificate_GetPublicKey
 }
 
 NSS_IMPLEMENT NSSPrivateKey *
-NSSCertificate_FindPrivateKey
-(
+NSSCertificate_FindPrivateKey (
   NSSCertificate *c,
   NSSCallback *uhh
 )
@@ -630,20 +721,30 @@ NSSCertificate_FindPrivateKey
 }
 
 NSS_IMPLEMENT PRBool
-NSSCertificate_IsPrivateKeyAvailable
-(
+NSSCertificate_IsPrivateKeyAvailable (
   NSSCertificate *c,
   NSSCallback *uhh,
   PRStatus *statusOpt
 )
 {
-    nss_SetError(NSS_ERROR_NOT_FOUND);
-    return PR_FALSE;
+    PRBool isUser = PR_FALSE;
+    nssCryptokiObject **ip;
+    nssCryptokiObject **instances = nssPKIObject_GetInstances(&c->object);
+    if (!instances) {
+	return PR_FALSE;
+    }
+    for (ip = instances; *ip; ip++) {
+	nssCryptokiObject *instance = *ip;
+	if (nssToken_IsPrivateKeyAvailable(instance->token, c, instance)) {
+	    isUser = PR_TRUE;
+	}
+    }
+    nssCryptokiObjectArray_Destroy(instances);
+    return isUser;
 }
 
 NSS_IMPLEMENT PRBool
-NSSUserCertificate_IsStillPresent
-(
+NSSUserCertificate_IsStillPresent (
   NSSUserCertificate *uc,
   PRStatus *statusOpt
 )
@@ -653,8 +754,7 @@ NSSUserCertificate_IsStillPresent
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSUserCertificate_Decrypt
-(
+NSSUserCertificate_Decrypt (
   NSSUserCertificate *uc,
   NSSAlgorithmAndParameters *apOpt,
   NSSItem *data,
@@ -671,8 +771,7 @@ NSSUserCertificate_Decrypt
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSUserCertificate_Sign
-(
+NSSUserCertificate_Sign (
   NSSUserCertificate *uc,
   NSSAlgorithmAndParameters *apOpt,
   NSSItem *data,
@@ -689,8 +788,7 @@ NSSUserCertificate_Sign
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSUserCertificate_SignRecover
-(
+NSSUserCertificate_SignRecover (
   NSSUserCertificate *uc,
   NSSAlgorithmAndParameters *apOpt,
   NSSItem *data,
@@ -707,8 +805,7 @@ NSSUserCertificate_SignRecover
 }
 
 NSS_IMPLEMENT NSSSymmetricKey *
-NSSUserCertificate_UnwrapSymmetricKey
-(
+NSSUserCertificate_UnwrapSymmetricKey (
   NSSUserCertificate *uc,
   NSSAlgorithmAndParameters *apOpt,
   NSSItem *wrappedKey,
@@ -725,8 +822,7 @@ NSSUserCertificate_UnwrapSymmetricKey
 }
 
 NSS_IMPLEMENT NSSSymmetricKey *
-NSSUserCertificate_DeriveSymmetricKey
-(
+NSSUserCertificate_DeriveSymmetricKey (
   NSSUserCertificate *uc, /* provides private key */
   NSSCertificate *c, /* provides public key */
   NSSAlgorithmAndParameters *apOpt,
@@ -741,8 +837,7 @@ NSSUserCertificate_DeriveSymmetricKey
 }
 
 NSS_IMPLEMENT void 
-nssBestCertificate_SetArgs
-(
+nssBestCertificate_SetArgs (
   nssBestCertificateCB *best,
   NSSTime *timeOpt,
   NSSUsage *usage,
@@ -761,8 +856,7 @@ nssBestCertificate_SetArgs
 }
 
 NSS_IMPLEMENT PRStatus 
-nssBestCertificate_Callback
-(
+nssBestCertificate_Callback (
   NSSCertificate *c, 
   void *arg
 )
@@ -782,7 +876,9 @@ nssBestCertificate_Callback
 	     * what the trust values are for the cert.
 	     * Ignore the returned pointer, the refcount is in c anyway.
 	     */
-	    (void)STAN_GetCERTCertificate(c);
+	    if (STAN_GetCERTCertificate(c) == NULL) {
+		return PR_FAILURE;
+	    }
 #endif
 	    if (dc->matchUsage(dc, best->usage)) {
 		best->cert = nssCertificate_AddRef(c);
@@ -818,29 +914,30 @@ nssBestCertificate_Callback
 }
 
 NSS_IMPLEMENT nssSMIMEProfile *
-nssSMIMEProfile_Create
-(
+nssSMIMEProfile_Create (
   NSSCertificate *cert,
   NSSItem *profileTime,
   NSSItem *profileData
 )
 {
-    PRStatus nssrv;
     NSSArena *arena;
     nssSMIMEProfile *rvProfile;
+    nssPKIObject *object;
+    NSSTrustDomain *td = nssCertificate_GetTrustDomain(cert);
+    NSSCryptoContext *cc = nssCertificate_GetCryptoContext(cert);
     arena = nssArena_Create();
     if (!arena) {
 	return NULL;
     }
-    rvProfile = nss_ZNEW(arena, nssSMIMEProfile);
-    if (!rvProfile) {
-	nssArena_Destroy(arena);
-	return NULL;
-    }
-    nssrv = nssPKIObject_Initialize(&rvProfile->object, arena, NULL, NULL);
-    if (nssrv != PR_SUCCESS) {
+    object = nssPKIObject_Create(arena, NULL, td, cc);
+    if (!object) {
 	goto loser;
     }
+    rvProfile = nss_ZNEW(arena, nssSMIMEProfile);
+    if (!rvProfile) {
+	goto loser;
+    }
+    rvProfile->object = *object;
     rvProfile->certificate = cert;
     rvProfile->email = nssUTF8_Duplicate(cert->email, arena);
     rvProfile->subject = nssItem_Duplicate(&cert->subject, arena, NULL);
@@ -852,14 +949,13 @@ nssSMIMEProfile_Create
     }
     return rvProfile;
 loser:
-    nssPKIObject_Destroy(&rvProfile->object);
-    return NULL;
+    nssPKIObject_Destroy(object);
+    return (nssSMIMEProfile *)NULL;
 }
 
 /* execute a callback function on all members of a cert list */
 NSS_EXTERN PRStatus
-nssCertificateList_DoCallback
-(
+nssCertificateList_DoCallback (
   nssList *certList, 
   PRStatus (* callback)(NSSCertificate *c, void *arg),
   void *arg
@@ -887,8 +983,7 @@ static PRStatus add_ref_callback(NSSCertificate *c, void *a)
 }
 
 NSS_IMPLEMENT void
-nssCertificateList_AddReferences
-(
+nssCertificateList_AddReferences (
   nssList *certList
 )
 {
@@ -896,8 +991,83 @@ nssCertificateList_AddReferences
 }
 
 NSS_IMPLEMENT NSSTrust *
-nssTrust_AddRef
-(
+nssTrust_Create (
+  nssPKIObject *object,
+  NSSItem *certData
+)
+{
+    PRStatus status;
+    PRUint32 i;
+    PRUint32 lastTrustOrder, myTrustOrder;
+    unsigned char sha1_hashcmp[SHA1_LENGTH];
+    unsigned char sha1_hashin[SHA1_LENGTH];
+    NSSItem sha1_hash;
+    NSSTrust *rvt;
+    nssCryptokiObject *instance;
+    nssTrustLevel serverAuth, clientAuth, codeSigning, emailProtection;
+    SECStatus rv; /* Should be stan flavor */
+    lastTrustOrder = 1<<16; /* just make it big */
+    PR_ASSERT(object->instances != NULL && object->numInstances > 0);
+    rvt = nss_ZNEW(object->arena, NSSTrust);
+    if (!rvt) {
+	return (NSSTrust *)NULL;
+    }
+    rvt->object = *object;
+
+    /* should be stan flavor of Hashbuf */
+    rv = PK11_HashBuf(SEC_OID_SHA1,sha1_hashcmp,certData->data,certData->size);
+    if (rv != SECSuccess) {
+	return (NSSTrust *)NULL;
+    }
+    sha1_hash.data = sha1_hashin;
+    sha1_hash.size = sizeof (sha1_hashin);
+    /* trust has to peek into the base object members */
+    PZ_Lock(object->lock);
+    for (i=0; i<object->numInstances; i++) {
+	instance = object->instances[i];
+	myTrustOrder = nssToken_GetTrustOrder(instance->token);
+	status = nssCryptokiTrust_GetAttributes(instance, NULL,
+						&sha1_hash,
+	                                        &serverAuth,
+	                                        &clientAuth,
+	                                        &codeSigning,
+	                                        &emailProtection);
+	if (status != PR_SUCCESS) {
+	    PZ_Unlock(object->lock);
+	    return (NSSTrust *)NULL;
+	}
+	if (PORT_Memcmp(sha1_hashin,sha1_hashcmp,SHA1_LENGTH) != 0) {
+	    PZ_Unlock(object->lock);
+	    return (NSSTrust *)NULL;
+	}
+	if (rvt->serverAuth == nssTrustLevel_Unknown ||
+	    myTrustOrder < lastTrustOrder) 
+	{
+	    rvt->serverAuth = serverAuth;
+	}
+	if (rvt->clientAuth == nssTrustLevel_Unknown ||
+	    myTrustOrder < lastTrustOrder) 
+	{
+	    rvt->clientAuth = clientAuth;
+	}
+	if (rvt->emailProtection == nssTrustLevel_Unknown ||
+	    myTrustOrder < lastTrustOrder) 
+	{
+	    rvt->emailProtection = emailProtection;
+	}
+	if (rvt->codeSigning == nssTrustLevel_Unknown ||
+	    myTrustOrder < lastTrustOrder) 
+	{
+	    rvt->codeSigning = codeSigning;
+	}
+	lastTrustOrder = myTrustOrder;
+    }
+    PZ_Unlock(object->lock);
+    return rvt;
+}
+
+NSS_IMPLEMENT NSSTrust *
+nssTrust_AddRef (
   NSSTrust *trust
 )
 {
@@ -908,8 +1078,7 @@ nssTrust_AddRef
 }
 
 NSS_IMPLEMENT PRStatus
-nssTrust_Destroy
-(
+nssTrust_Destroy (
   NSSTrust *trust
 )
 {
@@ -920,8 +1089,7 @@ nssTrust_Destroy
 }
 
 NSS_IMPLEMENT nssSMIMEProfile *
-nssSMIMEProfile_AddRef
-(
+nssSMIMEProfile_AddRef (
   nssSMIMEProfile *profile
 )
 {
@@ -932,8 +1100,7 @@ nssSMIMEProfile_AddRef
 }
 
 NSS_IMPLEMENT PRStatus
-nssSMIMEProfile_Destroy
-(
+nssSMIMEProfile_Destroy (
   nssSMIMEProfile *profile
 )
 {
@@ -943,3 +1110,74 @@ nssSMIMEProfile_Destroy
     return PR_SUCCESS;
 }
 
+NSS_IMPLEMENT NSSCRL *
+nssCRL_Create (
+  nssPKIObject *object
+)
+{
+    PRStatus status;
+    NSSCRL *rvCRL;
+    NSSArena *arena = object->arena;
+    PR_ASSERT(object->instances != NULL && object->numInstances > 0);
+    rvCRL = nss_ZNEW(arena, NSSCRL);
+    if (!rvCRL) {
+	return (NSSCRL *)NULL;
+    }
+    rvCRL->object = *object;
+    /* XXX should choose instance based on some criteria */
+    status = nssCryptokiCRL_GetAttributes(object->instances[0],
+                                          NULL,  /* XXX sessionOpt */
+                                          arena,
+                                          &rvCRL->encoding,
+                                          NULL, /* subject */
+                                          NULL, /* class */
+                                          &rvCRL->url,
+                                          &rvCRL->isKRL);
+    if (status != PR_SUCCESS) {
+	return (NSSCRL *)NULL;
+    }
+    return rvCRL;
+}
+
+NSS_IMPLEMENT NSSCRL *
+nssCRL_AddRef (
+  NSSCRL *crl
+)
+{
+    if (crl) {
+	nssPKIObject_AddRef(&crl->object);
+    }
+    return crl;
+}
+
+NSS_IMPLEMENT PRStatus
+nssCRL_Destroy (
+  NSSCRL *crl
+)
+{
+    if (crl) {
+	(void)nssPKIObject_Destroy(&crl->object);
+    }
+    return PR_SUCCESS;
+}
+
+NSS_IMPLEMENT PRStatus
+nssCRL_DeleteStoredObject (
+  NSSCRL *crl,
+  NSSCallback *uhh
+)
+{
+    return nssPKIObject_DeleteStoredObject(&crl->object, uhh, PR_TRUE);
+}
+
+NSS_IMPLEMENT NSSDER *
+nssCRL_GetEncoding (
+  NSSCRL *crl
+)
+{
+    if (crl->encoding.data != NULL && crl->encoding.size > 0) {
+	return &crl->encoding;
+    } else {
+	return (NSSDER *)NULL;
+    }
+}
