@@ -32,7 +32,7 @@
  */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: pkistore.c,v $ $Revision: 1.11.4.1 $ $Date: 2002/03/08 22:18:44 $ $Name:  $";
+static const char CVS_ID[] = "@(#) $RCSfile: pkistore.c,v $ $Revision: 1.22 $ $Date: 2003/01/08 21:48:45 $ $Name: MOZILLA_1_4_1_RELEASE $";
 #endif /* DEBUG */
 
 #ifndef PKIM_H
@@ -104,8 +104,7 @@ static PRIntn subject_list_sort(void *v1, void *v2)
 }
 
 NSS_IMPLEMENT nssCertificateStore *
-nssCertificateStore_Create
-(
+nssCertificateStore_Create (
   NSSArena *arenaOpt
 )
 {
@@ -161,12 +160,17 @@ loser:
     return NULL;
 }
 
-NSS_IMPLEMENT void
-nssCertificateStore_Destroy
-(
+extern const NSSError NSS_ERROR_BUSY;
+
+NSS_IMPLEMENT PRStatus
+nssCertificateStore_Destroy (
   nssCertificateStore *store
 )
 {
+    if (nssHash_Count(store->issuer_and_serial) > 0) {
+	nss_SetError(NSS_ERROR_BUSY);
+	return PR_FAILURE;
+    }
     PZ_DestroyLock(store->lock);
     nssHash_Destroy(store->issuer_and_serial);
     nssHash_Destroy(store->subject);
@@ -175,11 +179,11 @@ nssCertificateStore_Destroy
     } else {
 	nss_ZFreeIf(store);
     }
+    return PR_SUCCESS;
 }
 
 static PRStatus
-add_certificate_entry
-(
+add_certificate_entry (
   nssCertificateStore *store,
   NSSCertificate *cert
 )
@@ -199,8 +203,7 @@ add_certificate_entry
 }
 
 static PRStatus
-add_subject_entry
-(
+add_subject_entry (
   nssCertificateStore *store,
   NSSCertificate *cert
 )
@@ -231,15 +234,13 @@ add_subject_entry
 
 /* declared below */
 static void
-remove_certificate_entry
-(
+remove_certificate_entry (
   nssCertificateStore *store,
   NSSCertificate *cert
 );
 
 NSS_IMPLEMENT PRStatus
-nssCertificateStore_Add
-(
+nssCertificateStore_Add (
   nssCertificateStore *store,
   NSSCertificate *cert
 )
@@ -253,9 +254,7 @@ nssCertificateStore_Add
     nssrv = add_certificate_entry(store, cert);
     if (nssrv == PR_SUCCESS) {
 	nssrv = add_subject_entry(store, cert);
-	if (nssrv == PR_SUCCESS) {
-	    nssCertificate_AddRef(cert); /* obtain a reference for the store */
-	} else {
+	if (nssrv == PR_FAILURE) {
 	    remove_certificate_entry(store, cert);
 	}
     }
@@ -264,8 +263,7 @@ nssCertificateStore_Add
 }
 
 static void
-remove_certificate_entry
-(
+remove_certificate_entry (
   nssCertificateStore *store,
   NSSCertificate *cert
 )
@@ -286,8 +284,7 @@ remove_certificate_entry
 }
 
 static void
-remove_subject_entry
-(
+remove_subject_entry (
   nssCertificateStore *store,
   NSSCertificate *cert
 )
@@ -314,53 +311,38 @@ remove_subject_entry
 }
 
 NSS_IMPLEMENT void
-nssCertificateStore_Remove
-(
+nssCertificateStore_RemoveCertLOCKED (
   nssCertificateStore *store,
   NSSCertificate *cert
 )
 {
     certificate_hash_entry *entry;
-    PZ_Lock(store->lock);
-#ifdef NSS_3_4_CODE
-    if (cert->object.refCount > 2) {
-	/* This continues the hack described in CERT_DestroyCertificate.
-	 * Because NSS 3.4 maintains a single, global, crypto context,
-	 * certs must be explicitly removed from it when there are no
-	 * more references to them.  This is done by destroying the cert
-	 * when there are two references left, the one being destroyed,
-	 * and the one here (read: temp db).
-	 * However, there is a race condition with timing the removal
-	 * of the cert from the temp store and deleting the last
-	 * reference.  In CERT_DestroyCertificate, the refCount is checked,
-	 * and if it is two, a call is made here to remove the temp cert.
-	 * But by the time it gets here (and within the safety of the
-	 * store's lock), another thread could have grabbed a reference
-	 * to it.  Removing it now will wreak havoc.
-	 * Therefore, it is necessary to check the refCount again, 
-	 * after obtaining the store's lock, to make sure the cert is
-	 * actually ready to be deleted.  This check is safe, because
-	 * within the store's lock a cert that has only two references
-	 * *must* have one in the store, and the one being deleted.
-	 * See bug 125263.
-	 */
-	PZ_Unlock(store->lock);
-	return;
-    }
-#endif
     entry = (certificate_hash_entry *)
                               nssHash_Lookup(store->issuer_and_serial, cert);
     if (entry && entry->cert == cert) {
 	remove_certificate_entry(store, cert);
 	remove_subject_entry(store, cert);
-	NSSCertificate_Destroy(cert); /* release the store's reference */
     }
+}
+
+NSS_IMPLEMENT void
+nssCertificateStore_Lock (
+  nssCertificateStore *store
+)
+{
+    PZ_Lock(store->lock);
+}
+
+NSS_IMPLEMENT void
+nssCertificateStore_Unlock (
+  nssCertificateStore *store
+)
+{
     PZ_Unlock(store->lock);
 }
 
 static NSSCertificate **
-get_array_from_list
-(
+get_array_from_list (
   nssList *certList,
   NSSCertificate *rvOpt[],
   PRUint32 maximumOpt,
@@ -388,8 +370,7 @@ get_array_from_list
 }
 
 NSS_IMPLEMENT NSSCertificate **
-nssCertificateStore_FindCertificatesBySubject
-(
+nssCertificateStore_FindCertificatesBySubject (
   nssCertificateStore *store,
   NSSDER *subject,
   NSSCertificate *rvOpt[],
@@ -435,8 +416,8 @@ static void match_nickname(const void *k, void *v, void *a)
     nssList *subjectList = (nssList *)v;
     struct nickname_template_str *nt = (struct nickname_template_str *)a;
     nssrv = nssList_GetArray(subjectList, (void **)&c, 1);
-    nickname = NSSCertificate_GetNickname(c, NULL);
-    if (nssrv == PR_SUCCESS && 
+    nickname = nssCertificate_GetNickname(c, NULL);
+    if (nssrv == PR_SUCCESS && nickname &&
          nssUTF8_Equal(nickname, nt->nickname, &nssrv)) 
     {
 	nt->subjectList = subjectList;
@@ -447,8 +428,7 @@ static void match_nickname(const void *k, void *v, void *a)
  * Find all cached certs with this label.
  */
 NSS_IMPLEMENT NSSCertificate **
-nssCertificateStore_FindCertificatesByNickname
-(
+nssCertificateStore_FindCertificatesByNickname (
   nssCertificateStore *store,
   NSSUTF8 *nickname,
   NSSCertificate *rvOpt[],
@@ -505,8 +485,7 @@ static void match_email(const void *k, void *v, void *a)
  * Find all cached certs with this email address.
  */
 NSS_IMPLEMENT NSSCertificate **
-nssCertificateStore_FindCertificatesByEmail
-(
+nssCertificateStore_FindCertificatesByEmail (
   nssCertificateStore *store,
   NSSASCII7 *email,
   NSSCertificate *rvOpt[],
@@ -537,8 +516,7 @@ nssCertificateStore_FindCertificatesByEmail
 }
 
 NSS_IMPLEMENT NSSCertificate *
-nssCertificateStore_FindCertificateByIssuerAndSerialNumber
-(
+nssCertificateStore_FindCertificateByIssuerAndSerialNumber (
   nssCertificateStore *store,
   NSSDER *issuer,
   NSSDER *serial
@@ -561,8 +539,7 @@ nssCertificateStore_FindCertificateByIssuerAndSerialNumber
 
 #ifdef NSS_3_4_CODE
 static PRStatus
-issuer_and_serial_from_encoding
-(
+issuer_and_serial_from_encoding (
   NSSBER *encoding, 
   NSSDER *issuer, 
   NSSDER *serial
@@ -590,8 +567,7 @@ issuer_and_serial_from_encoding
 #endif
 
 NSS_IMPLEMENT NSSCertificate *
-nssCertificateStore_FindCertificateByEncodedCertificate
-(
+nssCertificateStore_FindCertificateByEncodedCertificate (
   nssCertificateStore *store,
   NSSDER *encoding
 )
@@ -616,8 +592,7 @@ nssCertificateStore_FindCertificateByEncodedCertificate
 }
 
 NSS_EXTERN PRStatus
-nssCertificateStore_AddTrust
-(
+nssCertificateStore_AddTrust (
   nssCertificateStore *store,
   NSSTrust *trust
 )
@@ -636,8 +611,7 @@ nssCertificateStore_AddTrust
 }
 
 NSS_IMPLEMENT NSSTrust *
-nssCertificateStore_FindTrustForCertificate
-(
+nssCertificateStore_FindTrustForCertificate (
   nssCertificateStore *store,
   NSSCertificate *cert
 )
@@ -655,8 +629,7 @@ nssCertificateStore_FindTrustForCertificate
 }
 
 NSS_EXTERN PRStatus
-nssCertificateStore_AddSMIMEProfile
-(
+nssCertificateStore_AddSMIMEProfile (
   nssCertificateStore *store,
   nssSMIMEProfile *profile
 )
@@ -675,8 +648,7 @@ nssCertificateStore_AddSMIMEProfile
 }
 
 NSS_IMPLEMENT nssSMIMEProfile *
-nssCertificateStore_FindSMIMEProfileForCertificate
-(
+nssCertificateStore_FindSMIMEProfileForCertificate (
   nssCertificateStore *store,
   NSSCertificate *cert
 )
@@ -696,12 +668,11 @@ nssCertificateStore_FindSMIMEProfileForCertificate
 /* XXX this is also used by cache and should be somewhere else */
 
 static PLHashNumber
-nss_certificate_hash
-(
+nss_certificate_hash (
   const void *key
 )
 {
-    int i;
+    unsigned int i;
     PLHashNumber h;
     NSSCertificate *c = (NSSCertificate *)key;
     h = 0;
@@ -723,8 +694,7 @@ nss_compare_certs(const void *v1, const void *v2)
 }
 
 NSS_IMPLEMENT nssHash *
-nssHash_CreateCertificate
-(
+nssHash_CreateCertificate (
   NSSArena *arenaOpt,
   PRUint32 numBuckets
 )
@@ -737,8 +707,7 @@ nssHash_CreateCertificate
 }
 
 NSS_IMPLEMENT void
-nssCertificateStore_DumpStoreInfo
-(
+nssCertificateStore_DumpStoreInfo (
   nssCertificateStore *store,
   void (* cert_dump_iter)(const void *, void *, void *),
   void *arg
