@@ -40,9 +40,9 @@
 #include "nsIDOMDocument.h"
 #include "nsIURL.h"
 #include "nsIChannel.h"
+#include "nsNetUtil.h"
 #include "nsIDOMWindowInternal.h"
 #include "nsIFilePicker.h"
-#include "nsIFileURL.h"
 #include "nsILookAndFeel.h"
 #include "nsIComponentManager.h"
 #include "nsIFactory.h"
@@ -76,6 +76,7 @@
 #include "nsGUIEvent.h"
 #include "nsIWebProgress.h"
 #include "nsIWebBrowserSetup.h"
+#include "nsIWebBrowserPrint.h"
 
 #include "nsCWebBrowser.h"
 #include "nsUnicharUtils.h"
@@ -155,6 +156,7 @@ static NS_DEFINE_CID(kWalletServiceCID, NS_WALLETSERVICE_CID);
 #endif
 #define BUTTON_WIDTH 90
 #define BUTTON_HEIGHT THROBBER_HEIGHT
+#define MAX_TEXT_LENGTH 30000
 
 #ifdef INSET_WEBSHELL
 #define WEBSHELL_LEFT_INSET 5
@@ -396,6 +398,18 @@ NS_IMETHODIMP nsBrowserWindow::SetEnabled(PRBool aEnabled)
   return NS_OK;
 }
 
+NS_IMETHODIMP nsBrowserWindow::GetBlurSuppression(PRBool *aBlurSuppression)
+{
+  NS_ENSURE_ARG_POINTER(aBlurSuppression);
+  *aBlurSuppression = PR_FALSE;
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP nsBrowserWindow::SetBlurSuppression(PRBool aBlurSuppression)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 NS_IMETHODIMP nsBrowserWindow::GetMainWidget(nsIWidget** aMainWidget)
 {
    NS_ENSURE_ARG_POINTER(aMainWidget);
@@ -408,7 +422,7 @@ NS_IMETHODIMP nsBrowserWindow::GetMainWidget(nsIWidget** aMainWidget)
 NS_IMETHODIMP nsBrowserWindow::SetFocus()
 {
    //XXX First Check In
-   NS_ASSERTION(PR_FALSE, "Not Yet Implemented");
+   NS_WARNING("Not Yet Implemented");
    return NS_OK;
 }
 
@@ -1137,16 +1151,15 @@ nsBrowserWindow::DoFileOpen()
 
   if (GetFileFromFileSelector(parentWindow, getter_AddRefs(file),
                               getter_AddRefs(mOpenFileDirectory))) {
-    nsCOMPtr<nsIFileURL> fileURL = do_CreateInstance(NS_STANDARDURL_CONTRACTID);
-    if (fileURL) {
-      fileURL->SetFile(file);
-
-      nsCAutoString url;
-      fileURL->GetSpec(url);
+    nsCOMPtr<nsIURI> uri;
+    NS_NewFileURI(getter_AddRefs(uri), file);
+    if (uri) {
+      nsCAutoString spec;
+      uri->GetSpec(spec);
       
       // Ask the Web widget to load the file URL
       nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(mWebBrowser));
-      webNav->LoadURI(NS_ConvertUTF8toUCS2(url).get(),
+      webNav->LoadURI(NS_ConvertUTF8toUCS2(spec).get(),
                       nsIWebNavigation::LOAD_FLAGS_NONE,
                       nsnull,
                       nsnull,
@@ -1321,7 +1334,6 @@ GetTitleSuffix(void)
 // Note: operator new zeros our memory
 nsBrowserWindow::nsBrowserWindow()
 {
-  NS_INIT_REFCNT();
   if (!gTitleSuffix) {
 #if XXX
     gTitleSuffix = GetTitleSuffix();
@@ -1663,6 +1675,7 @@ nsBrowserWindow::CreateToolBar(PRInt32 aWidth)
     PRUint32 size;
     nsAutoString empty;
     mLocation->SetText(empty, size);
+    mLocation->SetMaxTextLength(MAX_TEXT_LENGTH);
     NS_RELEASE(widget);
   }
 
@@ -1722,6 +1735,7 @@ nsBrowserWindow::CreateStatusBar(PRInt32 aWidth)
     widget->SetForegroundColor(NS_RGB(0, 0, 0));
     PRUint32 size;
     mStatus->SetText(nsAutoString(),size);
+    mStatus->SetMaxTextLength(MAX_TEXT_LENGTH);
 
     nsITextWidget* textWidget = nsnull;
     if (NS_OK == mStatus->QueryInterface(kITextWidgetIID,(void**)&textWidget))
@@ -2206,11 +2220,13 @@ void nsBrowserWindow::DoPrint(void)
 
   mDocShell->GetContentViewer(getter_AddRefs(viewer));
 
-  if (viewer)
-  {
-    nsCOMPtr<nsIContentViewerFile> viewerFile = do_QueryInterface(viewer);
-    if (viewerFile) {
-      //viewerFile->Print(PR_FALSE, nsnull, (nsIWebProgressListener*)nsnull);
+  if (viewer) {
+    nsCOMPtr<nsIWebBrowserPrint> webBrowserPrint = do_QueryInterface(viewer);
+    if (webBrowserPrint) {
+      if (!mPrintSettings) {
+        webBrowserPrint->GetGlobalPrintSettings(getter_AddRefs(mPrintSettings));
+      }
+      webBrowserPrint->Print(mPrintSettings, (nsIWebProgressListener*)nsnull);
     }
   }
 }
@@ -2398,7 +2414,6 @@ nsBrowserWindow::DoEditorTest(nsIDocShell *aDocShell, PRInt32 aCommandID)
 #include "nsIContent.h"
 #include "nsIFrame.h"
 #include "nsIFrameDebug.h"
-#include "nsIStyleContext.h"
 #include "nsIStyleSet.h"
 
 
@@ -2514,7 +2529,7 @@ DumpFramesRecurse(nsIDocShell* aDocShell, FILE* out, nsString *aFilterName)
         shell->GetPresContext(&presContext);
 
         nsIFrameDebug* fdbg;
-        if (NS_SUCCEEDED(root->QueryInterface(NS_GET_IID(nsIFrameDebug), (void**)&fdbg))) {
+        if (NS_SUCCEEDED(CallQueryInterface(root, &fdbg))) {
           fdbg->List(presContext, out, 0);
         }
         NS_IF_RELEASE(presContext);
@@ -2549,191 +2564,6 @@ nsBrowserWindow::DumpFrames(FILE* out, nsString *aFilterName)
 
 //----------------------------------------------------------------------
 
-#include "nsISizeOfHandler.h"
-#include "nsQuickSort.h"
-
-struct SizeReportEntry {
-  nsIAtom* mKey;
-  PRUint32 mCount;
-  PRUint32 mTotalSize;
-  PRUint32 mMinSize;
-  PRUint32 mMaxSize;
-};
-
-static void
-GatherSizeReportData(nsISizeOfHandler* aHandler,
-                     nsIAtom* aKey,
-                     PRUint32 aCount,
-                     PRUint32 aTotalSize,
-                     PRUint32 aMinSize,
-                     PRUint32 aMaxSize,
-                     void* arg)
-{
-  if (aKey) {
-    nsVoidArray* array = (nsVoidArray*) arg;
-    SizeReportEntry* entry = new SizeReportEntry();
-    entry->mKey = aKey;
-    entry->mCount = aCount;
-    entry->mTotalSize = aTotalSize;
-    entry->mMinSize = aMinSize;
-    entry->mMaxSize = aMaxSize;
-    array->AppendElement((void*) entry);
-  }
-}
-
-static int PR_CALLBACK CompareEntries(const void* ve1, const void* ve2, void* closure)
-{
-  SizeReportEntry* e1 = (SizeReportEntry*) ve1;
-  SizeReportEntry* e2 = (SizeReportEntry*) ve2;
-  if (e1->mKey == e2->mKey) {
-    return 0;
-  }
-  nsAutoString k1, k2;
-  e1->mKey->ToString(k1);
-  e2->mKey->ToString(k2);
-  if (k1 < k2) {
-    return -1;
-  }
-  return 1;
-}
-
-static SizeReportEntry*
-SortFrameSizeList(nsVoidArray& sizeReportData)
-{
-  // Convert data in void array into a real array, freeing the old data
-  PRInt32 i, count = sizeReportData.Count();
-  SizeReportEntry* all = new SizeReportEntry[count];
-  SizeReportEntry* to = all;
-  for (i = 0; i < count; i++) {
-    SizeReportEntry* e = (SizeReportEntry*) sizeReportData[i];
-    *to = *e;
-    to++;
-    sizeReportData.ReplaceElementAt(nsnull, i);
-    delete e;
-  }
-
-  // Now sort the array
-  NS_QuickSort(all, count, sizeof(SizeReportEntry), CompareEntries, nsnull);
-
-  return all;
-}
-
-static void ShowReport(FILE* out, nsISizeOfHandler* aHandler)
-{
-  nsVoidArray sizeReportData;
-  aHandler->Report(GatherSizeReportData, (void*) &sizeReportData);
-
-  if (sizeReportData.Count()) {
-    PRInt32 count = sizeReportData.Count();
-    SizeReportEntry* entries = SortFrameSizeList(sizeReportData);
-
-    // Display headers
-    fprintf(out, "%-20s %-5s %-9s %-7s %-7s %-7s\n",
-            "Frame Type", "Count", "TotalSize",
-            "MinSize", "MaxSize", "AvgSize");
-    fprintf(out, "%-20s %-5s %-9s %-7s %-7s %-7s\n",
-            "----------", "-----", "---------",
-            "-------", "-------", "-------");
-
-    SizeReportEntry* entry = entries;
-    SizeReportEntry* end = entries + count;
-    for (; entry < end; entry++) {
-      nsAutoString type;
-      entry->mKey->ToString(type);
-
-      fprintf(out, "%-20s %5d %9d %7d %7d %7d\n",
-              NS_LossyConvertUCS2toASCII(type).get(),
-              entry->mCount, entry->mTotalSize,
-              entry->mMinSize, entry->mMaxSize,
-              entry->mCount ? entry->mTotalSize / entry->mCount : 0);
-    }
-    delete entries;
-
-    // Display totals and trailer
-    PRUint32 totalCount, totalSize;
-    aHandler->GetTotals(&totalCount, &totalSize);
-    fprintf(out, "%-20s %5d %9d\n",
-            "*** Total ***", totalCount, totalSize);
-  }
-}
-
-//----------------------------------------
-
-static void
-GatherFrameDataSizes(nsIPresContext* aPresContext, nsISizeOfHandler* aHandler, nsIFrame* aFrame)
-{
-  if (aFrame) {
-    // Add in the frame
-    nsCOMPtr<nsIAtom> frameType;
-    aFrame->GetFrameType(getter_AddRefs(frameType));
-    PRUint32 frameDataSize;
-    nsIFrameDebug* fdbg;
-    if (NS_SUCCEEDED(aFrame->QueryInterface(NS_GET_IID(nsIFrameDebug), (void**) &fdbg))) {
-      fdbg->SizeOf(aHandler, &frameDataSize);
-      aHandler->AddSize(frameType, frameDataSize);
-    }
-
-    // And all of its children
-    PRInt32 listIndex = 0;
-    nsIAtom* listName = nsnull;
-    do {
-      nsIFrame* kid;
-      aFrame->FirstChild(aPresContext, listName, &kid);
-      while (kid) {
-        GatherFrameDataSizes(aPresContext, aHandler, kid);
-        kid->GetNextSibling(&kid);
-      }
-      NS_IF_RELEASE(listName);
-      aFrame->GetAdditionalChildListName(listIndex, &listName);
-      listIndex++;
-    } while (nsnull != listName);
-  }
-}
-
-static void
-GatherFrameDataSizes(nsISizeOfHandler* aHandler, nsIDocShell* aDocShell)
-{
-  if (nsnull != aDocShell) {
-    nsIPresShell* shell = GetPresShellFor(aDocShell);
-    if (nsnull != shell) {
-      nsCOMPtr<nsIPresContext> presContext;
-      nsIFrame* root;
-      shell->GetRootFrame(&root);
-      shell->GetPresContext(getter_AddRefs(presContext));
-      if (nsnull != root) {
-        GatherFrameDataSizes(presContext, aHandler, root);
-      }
-      NS_RELEASE(shell);
-    }
-
-    // dump the frames of the sub documents
-    PRInt32 i, n;
-    nsCOMPtr<nsIDocShellTreeNode> docShellAsNode(do_QueryInterface(aDocShell));
-    docShellAsNode->GetChildCount(&n);
-    for (i = 0; i < n; i++) {
-      nsCOMPtr<nsIDocShellTreeItem> child;
-      docShellAsNode->GetChildAt(i, getter_AddRefs(child));
-      nsCOMPtr<nsIDocShell> childAsShell(do_QueryInterface(child));
-      if (childAsShell) {
-        GatherFrameDataSizes(aHandler, childAsShell);
-      }
-    }
-  }
-}
-
-void
-nsBrowserWindow::ShowFrameSize(FILE* out)
-{
-  nsCOMPtr<nsISizeOfHandler> handler;
-  nsresult rv = NS_NewSizeOfHandler(getter_AddRefs(handler));
-  if (NS_SUCCEEDED(rv) && handler) {
-    GatherFrameDataSizes(handler, mDocShell);
-    ShowReport(out, handler);
-  }
-}
-
-//----------------------------------------------------------------------
-
 static
 void
 DumpViewsRecurse(nsIDocShell* aDocShell, FILE* out)
@@ -2813,15 +2643,7 @@ nsBrowserWindow::DumpStyleContexts(FILE* out)
       if (nsnull == root) {
         fputs("null root frame\n", out);
       } else {
-        nsIStyleContext* rootContext;
-        root->GetStyleContext(&rootContext);
-        if (nsnull != rootContext) {
-          styleSet->ListContexts(rootContext, out);
-          NS_RELEASE(rootContext);
-        }
-        else {
-          fputs("null root context", out);
-        }
+        styleSet->ListContexts(root, out);
       }
     }
     NS_RELEASE(shell);
@@ -2966,129 +2788,6 @@ nsBrowserWindow::GetStringPref(const char * aPrefName, nsString& aValue)
 }
 
 #ifdef NS_DEBUG
-
-//----------------------------------------------------------------------
-
-static void
-GatherContentDataSizes(nsISizeOfHandler* aHandler, nsIContent* aContent)
-{
-  if (aContent) {
-    // Add in the content
-    nsCOMPtr<nsIAtom> tag;
-    aContent->GetTag(*getter_AddRefs(tag));
-    if (!tag) {
-      tag = getter_AddRefs(NS_NewAtom(":content-no-tag"));
-    }
-    PRUint32 contentDataSize;
-    aContent->SizeOf(aHandler, &contentDataSize);
-    aHandler->AddSize(tag, contentDataSize);
-
-    // And all of its children
-    PRInt32 i, childCount;
-    aContent->ChildCount(childCount);
-    for (i = 0; i < childCount; i++) {
-      nsIContent* kid;
-      aContent->ChildAt(i, kid);
-      if (kid) {
-        GatherContentDataSizes(aHandler, kid);
-        NS_RELEASE(kid);
-      }
-    }
-  }
-}
-
-static void
-GatherContentDataSizes(nsISizeOfHandler* aHandler, nsIDocShell* aDocShell)
-{
-  if (nsnull != aDocShell) {
-    nsIPresShell* shell = GetPresShellFor(aDocShell);
-    if (nsnull != shell) {
-      nsIDocument* doc;
-      shell->GetDocument(&doc);
-      if (doc) {
-        nsCOMPtr<nsIContent> rootContent;
-        doc->GetRootContent(getter_AddRefs(rootContent));
-        if (rootContent) {
-          GatherContentDataSizes(aHandler, rootContent.get());
-        }
-        NS_RELEASE(doc);
-      }
-      NS_RELEASE(shell);
-    }
-
-    // dump the frames of the sub documents
-    PRInt32 i, n;
-    nsCOMPtr<nsIDocShellTreeNode> docShellAsNode(do_QueryInterface(aDocShell));
-    docShellAsNode->GetChildCount(&n);
-    for (i = 0; i < n; i++) {
-      nsCOMPtr<nsIDocShellTreeItem> child;
-      docShellAsNode->GetChildAt(i, getter_AddRefs(child));
-      nsCOMPtr<nsIDocShell> childAsShell(do_QueryInterface(child));
-      if (childAsShell) {
-        GatherContentDataSizes(aHandler, childAsShell);
-      }
-    }
-  }
-}
-
-void
-nsBrowserWindow::ShowContentSize(FILE* out)
-{
-  nsCOMPtr<nsISizeOfHandler> handler;
-  nsresult rv = NS_NewSizeOfHandler(getter_AddRefs(handler));
-  if (NS_SUCCEEDED(rv) && handler) {
-    GatherContentDataSizes(handler, mDocShell);
-    ShowReport(out, handler);
-  }
-}
-
-//----------------------------------------------------------------------
-
-static void
-GatherStyleDataSizes(nsISizeOfHandler* aHandler, nsIDocShell* aDocShell)
-{
-  if (nsnull != aDocShell) {
-    nsIPresShell* shell = GetPresShellFor(aDocShell);
-    if (nsnull != shell) {
-      nsIStyleSet *styleSet;
-      shell->GetStyleSet(&styleSet);
-      if(styleSet){
-        PRUint32 ssSize;
-        styleSet->ResetUniqueStyleItems();
-        // size of the style set
-        styleSet->SizeOf(aHandler, ssSize);
-        NS_RELEASE(styleSet);
-      }
-      nsIFrame* root;
-      shell->GetRootFrame(&root);
-      if (nsnull == root) {
-        puts("null root frame\n");
-      } else {
-        nsIStyleContext* rootContext;
-        root->GetStyleContext(&rootContext);
-        if (nsnull != rootContext) {
-          PRUint32 contextSize;
-          // get the sizes from the root context
-          rootContext->SizeOf(aHandler, contextSize);
-          NS_RELEASE(rootContext);
-        }
-      }
-      NS_RELEASE(shell);
-    }
-  }
-}
-
-void
-nsBrowserWindow::ShowStyleSize(FILE* out)
-{
-  nsCOMPtr<nsISizeOfHandler> handler;
-  nsresult rv = NS_NewSizeOfHandler(getter_AddRefs(handler));
-  if (NS_SUCCEEDED(rv) && handler) {
-    GatherStyleDataSizes(handler, mDocShell);
-    ShowReport(out, handler);
-  }
-}
-
 void
 nsBrowserWindow::DoDebugSave()
 {
@@ -3210,21 +2909,6 @@ nsBrowserWindow::DispatchDebugMenu(PRInt32 aID)
       result = nsEventStatus_eConsumeNoDefault;
       break;
 
-    case VIEWER_SHOW_CONTENT_SIZE:
-      ShowContentSize();
-      result = nsEventStatus_eConsumeNoDefault;
-      break;
-
-    case VIEWER_SHOW_FRAME_SIZE:
-      ShowFrameSize();
-      result = nsEventStatus_eConsumeNoDefault;
-      break;
-
-    case VIEWER_SHOW_STYLE_SIZE:
-      ShowStyleSize();
-      result = nsEventStatus_eConsumeNoDefault;
-      break;
-
     case VIEWER_SHOW_CONTENT_QUALITY:
       if (nsnull != mDocShell) {
         nsIPresShell   *ps = GetPresShellFor(mDocShell);
@@ -3280,7 +2964,7 @@ nsBrowserWindow::SetCompatibilityMode(PRUint32 aMode)
   if (pref) { 
     int32 prefInt = USE_DTD;
     if (STANDARD == aMode) {
-      prefInt = eCompatibility_Standard;
+      prefInt = eCompatibility_FullStandards;
     }
     else if (QUIRKS == aMode) {
       prefInt = eCompatibility_NavQuirks;
@@ -3402,4 +3086,13 @@ NS_IMETHODIMP nsBrowserWindow::EnsureWebBrowserChrome()
    return NS_OK;
 }
 
+nsresult nsBrowserWindow::GetWindow(nsIWidget** aWindow)
+{
+  if (aWindow && mWindow) {
+    *aWindow = mWindow.get();
+    NS_IF_ADDREF(*aWindow);
+    return NS_OK;
+  }
+  return NS_ERROR_FAILURE;
+}
 
