@@ -27,6 +27,7 @@
 #include "nsMemory.h"
 
 #include "nsLocalFileWin.h"
+#include "nsNativeCharsetUtils.h"
 
 #include "nsISimpleEnumerator.h"
 #include "nsIComponentManager.h"
@@ -53,6 +54,13 @@
 
 #include "nsAutoLock.h"
 
+// _mbsstr isn't declared in w32api headers but it's there in the libs
+#ifdef __MINGW32__
+extern "C" {
+unsigned char *_mbsstr( const unsigned char *str,
+                        const unsigned char *substr );
+};
+#endif
 
 //----------------------------------------------------------------------------
 // short cut resolver
@@ -65,7 +73,7 @@ public:
     virtual ~ShortcutResolver();
 
     nsresult Init();
-    nsresult Resolve(const unsigned short* in, char* out);
+    nsresult Resolve(const WCHAR* in, char* out);
 
 private:
     PRLock*       mLock;
@@ -124,7 +132,7 @@ ShortcutResolver::Init()
 
 // |out| must be an allocated buffer of size MAX_PATH
 nsresult 
-ShortcutResolver::Resolve(const unsigned short* in, char* out)
+ShortcutResolver::Resolve(const WCHAR* in, char* out)
 {
     nsAutoLock lock(mLock);
     
@@ -249,6 +257,32 @@ myLL_L2II(PRInt64 result, PRInt32 *hi, PRInt32 *lo )
     LL_L2I(*lo, a64);
 }
 
+static PRBool
+IsShortcut(const char* workingPath, int filePathLen)
+{
+    // XXX this is badly broken!!
+    // XXX consider "C:\FOO.LNK"
+    // XXX consider "C:\foo.lnkx\bar.lnk"
+
+    // check to see if it is shortcut, i.e., it has ".lnk" in it
+    unsigned char* dest = _mbsstr((unsigned char*)workingPath,
+                                  (unsigned char*)".lnk");
+    if (!dest)
+        return PR_FALSE;
+
+    // find index of ".lnk"
+    int result = (int)(dest - (unsigned char*)workingPath);
+
+    // if ".lnk" is not at the leaf of this path, we need to make sure the
+    // next char after ".lnk" is a '\\'. e.g. "c:\\foo.lnk\\a.html" is valid,
+    // whereas "c:\\foo.lnkx" is not.
+    if (result + 4 < filePathLen)
+    {
+        if (workingPath[result + 4] != '\\')
+            return PR_FALSE;
+    }
+    return PR_TRUE;
+}
 
 //-----------------------------------------------------------------------------
 // nsDirEnumerator
@@ -262,7 +296,6 @@ class nsDirEnumerator : public nsISimpleEnumerator
 
         nsDirEnumerator() : mDir(nsnull) 
         {
-            NS_INIT_REFCNT();
         }
 
         nsresult Init(nsILocalFile* parent) 
@@ -339,7 +372,7 @@ class nsDirEnumerator : public nsISimpleEnumerator
 
             *result = mNext;        // might return nsnull
             NS_IF_ADDREF(*result);
-            
+
             mNext = nsnull;
             return NS_OK;
         }
@@ -368,7 +401,6 @@ NS_IMPL_ISUPPORTS1(nsDirEnumerator, nsISimpleEnumerator);
 
 nsLocalFile::nsLocalFile()
 {
-    NS_INIT_REFCNT();
     mLastResolution = PR_FALSE;
     mFollowSymlinks = PR_FALSE;
     MakeDirty();
@@ -409,13 +441,6 @@ NS_IMPL_THREADSAFE_ISUPPORTS2(nsLocalFile, nsILocalFile, nsIFile)
 // nsLocalFile <private>
 //-----------------------------------------------------------------------------
 
-// This function resets any cached information about the file.
-void
-nsLocalFile::MakeDirty()
-{
-    mDirty = PR_TRUE;
-}
-
 // ResolvePath
 //  this function will walk the native path of |this| resolving any symbolic
 //  links found.  The new resulting path will be placed into mResolvedPath.
@@ -424,8 +449,12 @@ nsLocalFile::ResolvePath(const char* workingPath, PRBool resolveTerminal, char**
 {
 
     nsresult rv = NS_OK;
+    PRBool isDir = PR_FALSE;
     
-    if (strstr(workingPath, ".lnk") == nsnull)
+    // check to see if it is shortcut, i.e., it has ".lnk" in it
+    int filePathLen = strlen(workingPath);
+    PRBool isShortcut = IsShortcut(workingPath, filePathLen);
+    if (!isShortcut)
         return NS_ERROR_FILE_INVALID_PATH;
 
 #ifdef DEBUG_dougt
@@ -433,10 +462,14 @@ nsLocalFile::ResolvePath(const char* workingPath, PRBool resolveTerminal, char**
 #endif
 
     // Get the native path for |this|
-    char* filePath = (char*) nsMemory::Clone( workingPath, strlen(workingPath)+1 );
+    // allocate extra bytes incase we need to append '\\' and '\0' to the 
+    // workingPath, if it is just a drive letter and a colon  
+    char *filePath = (char *) nsMemory::Alloc(filePathLen+2);
 
-    if (filePath == nsnull)
+    if (!filePath)
         return NS_ERROR_NULL_POINTER;
+
+    memcpy(filePath, workingPath, filePathLen + 1);
     
     // We are going to walk the native file path
     // and stop at each slash.  For each partial
@@ -445,141 +478,126 @@ nsLocalFile::ResolvePath(const char* workingPath, PRBool resolveTerminal, char**
     // if it is, we will resolve it and continue
     // with that resolved path.
     
-    // Get the first slash.          
-    char* slash = strchr(filePath, '\\');
-            
-    if (!slash && filePath[0] != nsnull && filePath[1] == ':' && filePath[2] == '\0')
+    // Get the first slash.
+    unsigned char* slash = _mbschr((unsigned char*) filePath, '\\');
+     
+    if (slash == nsnull)
     {
-        // we have a drive letter and a colon (eg 'c:'
-        // this is resolve already
-        
-        int filePathLen = strlen(filePath);
-        char* rp = (char*) nsMemory::Alloc( filePathLen + 2 );
-        if (!rp)
-            return NS_ERROR_OUT_OF_MEMORY;
-        memcpy( rp, filePath, filePathLen );
-        rp[filePathLen] = '\\';
-        rp[filePathLen+1] = 0;
-        *resolvedPath = rp;
-        
-        nsMemory::Free(filePath);
-        return NS_OK;
-    }
-    else
-    {
-        nsMemory::Free(filePath);
-        return NS_ERROR_FILE_INVALID_PATH;
+        if (nsCRT::IsAsciiAlpha(filePath[0]) && filePath[1] == ':' && filePath[2] == '\0')
+        {
+            // we have a drive letter and a colon (eg 'c:'
+            // this is resolve already
+            filePath[filePathLen] = '\\';
+            filePath[filePathLen+1] = '\0';
+
+            *resolvedPath = filePath;
+            return NS_OK;
+        }
+        else
+        {
+            nsMemory::Free(filePath);
+            return NS_ERROR_FILE_INVALID_PATH;
+        }
     }
         
 
     // We really cant have just a drive letter as
     // a shortcut, so we will skip the first '\\'
-    slash = strchr(++slash, '\\');
-    
+    slash = _mbschr(++slash, '\\');
+
     while (slash || resolveTerminal)
     {
         // Change the slash into a null so that
         // we can use the partial path. It is is 
         // null, we know it is the terminal node.
-        
         if (slash)
         {
             *slash = '\0';
         }
+        else if (resolveTerminal)
+        {
+            // this is our last time in this loop.
+            // set loop condition to false
+            resolveTerminal = PR_FALSE;
+        }
         else
         {
-            if (resolveTerminal)
+            // something is wrong.  we should not have
+            // both slash being null and resolveTerminal 
+            // not set!
+            nsMemory::Free(filePath);
+            return NS_ERROR_NULL_POINTER;
+        }
+                
+        // check to see the file is a shortcut by the magic .lnk extension.
+        size_t offset = strlen(filePath) - 4;
+        if ((offset > 0) && (strnicmp( (filePath + offset), ".lnk", 4) == 0))
+        {
+            nsAutoString ucsBuf;
+            NS_CopyNativeToUnicode(nsDependentCString(filePath), ucsBuf);
+            char *temp = (char*) nsMemory::Alloc( MAX_PATH );
+            if (temp == nsnull)
             {
-                // this is our last time in this loop.
-                // set loop condition to false
-
-                resolveTerminal = PR_FALSE;
-            }
-            else
-            {
-                // something is wrong.  we should not have
-                // both slash being null and resolveTerminal 
-                // not set!
                 nsMemory::Free(filePath);
                 return NS_ERROR_NULL_POINTER;
             }
-        }
-                
-        WORD wsz[MAX_PATH];     // TODO, Make this dynamically allocated.
 
-        // check to see the file is a shortcut by the magic .lnk extension.
-        size_t offset = strlen(filePath) - 4;
-        if ((offset > 0) && (strncmp( (filePath + offset), ".lnk", 4) == 0))
-        {
-            MultiByteToWideChar(CP_ACP, 0, filePath, -1, wsz, MAX_PATH); 
-        }        
-        else
-        {
-            char linkStr[MAX_PATH];
-            strcpy(linkStr, filePath);
-            strcat(linkStr, ".lnk");
+            if (gResolver)
+                rv = gResolver->Resolve(ucsBuf.get(), temp);
+            else
+                rv = NS_ERROR_FAILURE;
 
-            // Ensure that the string is Unicode. 
-            MultiByteToWideChar(CP_ACP, 0, linkStr, -1, wsz, MAX_PATH); 
-        }        
-
-        char *temp = (char*) nsMemory::Alloc( MAX_PATH );
-        if (temp == nsnull)
-            return NS_ERROR_NULL_POINTER;
-
-        if (gResolver)
-            rv = gResolver->Resolve(wsz, temp);
-        else
-            rv = NS_ERROR_FAILURE;
-
-        if (NS_SUCCEEDED(rv))
-        {
-            // found a new path.
-            
-            // addend a slash on it since it does not come out of GetPath()
-            // with one only if it is a directory.  If it is not a directory
-            // and there is more to append, than we have a problem.
-            
-            struct stat st;
-            int statrv = stat(temp, &st);
-            
-            if (0 == statrv && (_S_IFDIR & st.st_mode))
-                strcat(temp, "\\");
-            
-            if (slash)
+            if (NS_SUCCEEDED(rv))
             {
-                // save where we left off.
-                char *carot= (temp + strlen(temp) -1 );
-                
-                // append all the stuff that we have not done.
-                strcat(temp, ++slash);
-                
-                slash = carot;
-            }
-
-            nsMemory::Free(filePath);
-            filePath = temp;
+                // found a new path.
             
-        }
+                // addend a slash on it since it does not come out of GetPath()
+                // with one only if it is a directory.  If it is not a directory
+                // and there is more to append, than we have a problem.
+            
+                struct stat st;
+                int statrv = stat(temp, &st);
+            
+                if (0 == statrv && (_S_IFDIR & st.st_mode))
+                {
+                    strcat(temp, "\\");
+                    isDir = PR_TRUE;
+                }
+            
+                if (slash)
+                {
+                    // save where we left off.
+                    char *carot= (temp + strlen(temp) -1 );
+                
+                    // append all the stuff that we have not done.
+                    _mbscat((unsigned char*)temp, ++slash);
+                
+                    slash = (unsigned char*)carot;
+                }
 
-        else
+                nsMemory::Free(filePath);
+                filePath = temp;
+            
+            }
+            else
+            {
+                // could not resolve shortcut.  Return error;
+                nsMemory::Free(filePath);
+                return NS_ERROR_FILE_INVALID_PATH;
+            }
+        }
+        if (slash)
         {
-            // could not resolve shortcut.  Return error;
-            nsMemory::Free(filePath);
-            return NS_ERROR_FILE_INVALID_PATH;
+            *slash = '\\';
+            ++slash;
+            slash = _mbschr(slash, '\\');
         }
     }    
-    if (slash)
-    {
-        *slash = '\\';
-        ++slash;
-        slash = strchr(slash, '\\');
-    }
 
     // kill any trailing separator
     char* temp = filePath;
     int len = strlen(temp) - 1;
-    if(temp[len] == '\\')
+    if((temp[len] == '\\') && (!isDir))
         temp[len] = '\0';
     
     *resolvedPath = filePath;
@@ -630,7 +648,7 @@ nsLocalFile::ResolveAndStat(PRBool resolveTerminal)
     
         // if we found the file and we are not following symlinks, then return success.
  
-        if (!mFollowSymlinks || pathLen < 4 || (strcmp(leaf, ".lnk") != 0))
+        if (!mFollowSymlinks || pathLen < 4 || (stricmp(leaf, ".lnk") != 0))
         {
 		    mDirty = PR_FALSE;
             return NS_OK;
@@ -725,7 +743,7 @@ nsLocalFile::InitWithNativePath(const nsACString &filePath)
     // kill any trailing '\' provided it isn't the second char of DBCS
     PRInt32 len = pathLen - 1;
     if (path[len] == '\\' && (!::IsDBCSLeadByte(path[len-1]) || 
-        _mbsrchr((const unsigned char *)path, '\\') == (const unsigned char *)path+len))
+                _mbsrchr((const unsigned char *)path, '\\') == (const unsigned char *)path+len))
     {
         path[len] = '\0';
         pathLen = len;
@@ -738,10 +756,15 @@ nsLocalFile::InitWithNativePath(const nsACString &filePath)
 NS_IMETHODIMP  
 nsLocalFile::OpenNSPRFileDesc(PRInt32 flags, PRInt32 mode, PRFileDesc **_retval)
 {
-    if (mDirty)
+    // check to see if it is shortcut, i.e., it has ".lnk" in it
+    PRBool isShortcut = IsShortcut(mWorkingPath.get(), mWorkingPath.Length());
+
+    if (!isShortcut && mDirty)
     {  
-        // we will optimize here.  If we are opening a file and we are still
-        // dirty, assume that the working path is vaild and try to open it.
+        // we will optimize here. If we are not a shortcut and we are opening
+        // a file and we are still dirty, assume that the working path is vaild
+        // and try to open it. The working path will be different from its
+        // resolved path for a shortcut file.
         // If it does work, get the stat info via the file descriptor 
         mResolvedPath.Assign(mWorkingPath);
         *_retval = PR_Open(mResolvedPath.get(), flags, mode);
@@ -799,8 +822,31 @@ nsLocalFile::Create(PRUint32 type, PRUint32 attributes)
     if (NS_FAILED(rv) && rv != NS_ERROR_FILE_NOT_FOUND)
         return rv;  
     
-   // create nested directories to target
-    unsigned char* slash = _mbschr((const unsigned char*) mResolvedPath.get(), '\\');
+    // create directories to target
+    //
+    // A given local file can be either one of these forms:
+    //
+    //   - normal:    X:\some\path\on\this\drive
+    //                       ^--- start here
+    //
+    //   - UNC path:  \\machine\volume\some\path\on\this\drive
+    //                                     ^--- start here
+    //
+    // Skip the first 'X:\' for the first form, and skip the first full
+    // '\\machine\volume\' segment for the second form.
+
+    const unsigned char* path = (const unsigned char*) mResolvedPath.get();
+    if (path[0] == '\\' && path[1] == '\\')
+    {
+        // dealing with a UNC path here; skip past '\\machine\'
+        path = _mbschr(path + 2, '\\');
+        if (!path)
+            return NS_ERROR_FILE_INVALID_PATH;
+        ++path;
+    }
+
+    // search for first slash after the drive (or volume) name
+    unsigned char* slash = _mbschr(path, '\\');
 
     if (slash)
     {
@@ -814,7 +860,12 @@ nsLocalFile::Create(PRUint32 type, PRUint32 attributes)
         
             if (!CreateDirectoryA(mResolvedPath.get(), NULL)) {
                 rv = ConvertWinError(GetLastError());
-                if (rv != NS_ERROR_FILE_ALREADY_EXISTS) return rv;
+                // perhaps the base path already exists, or perhaps we don't have
+                // permissions to create the directory.  NOTE: access denied could
+                // occur on a parent directory even though it exists.
+                if (rv != NS_ERROR_FILE_ALREADY_EXISTS &&
+                    rv != NS_ERROR_FILE_ACCESS_DENIED)
+                    return rv;
             }
             *slash = '\\';
             ++slash;
@@ -845,12 +896,14 @@ nsLocalFile::Create(PRUint32 type, PRUint32 attributes)
 NS_IMETHODIMP  
 nsLocalFile::AppendNative(const nsACString &node)
 {
+    if (node.IsEmpty())
+        return NS_OK;
+
     // Append only one component. Check for subdirs.
     // XXX can we avoid the PromiseFlatCString call?
-    if (node.IsEmpty() || (_mbschr((const unsigned char*) PromiseFlatCString(node).get(), '\\') != nsnull))
-    {
+    if (_mbschr((const unsigned char*) PromiseFlatCString(node).get(), '\\') != nsnull)
         return NS_ERROR_FILE_UNRECOGNIZED_PATH;
-    }
+
     return AppendRelativeNativePath(node);
 }
 
@@ -970,9 +1023,59 @@ nsLocalFile::CopySingleFile(nsIFile *sourceFile, nsIFile *destParent, const nsAC
 
     if (!move)
         copyOK = CopyFile(filePath.get(), destPath.get(), PR_TRUE);
-    else
+    else 
+    {
+        // What we have to do is check to see if the destPath exists.  If it 
+        // does, we have to move it out of the say so that MoveFile will 
+        // succeed.  However, we don't want to just remove it since MoveFile 
+        // can fail leaving us without a file.
+        
+        nsCAutoString backup;
+        PRFileInfo64 fileInfo64;
+        PRStatus status = PR_GetFileInfo64(destPath.get(), &fileInfo64);
+        if (status == PR_SUCCESS)
+        {
+        
+            // the file exists.  Check to make sure it is not a directory,
+            // then move it out of the way.
+            if (fileInfo64.type == PR_FILE_FILE)
+            {
+                backup.Append(destPath);
+                backup.Append(".moztmp");
+
+                // remove any existing backup file that we may already have.
+                // maybe we should be doing some kind of unique naming here, 
+                // but why bother.
+                remove(backup.get());
+                
+                // move destination file to backup file
+                copyOK = MoveFile(destPath.get(), backup.get());
+                if (!copyOK)
+                {
+                    // I guess we can't do the backup copy, so return.
+                    rv = ConvertWinError(GetLastError());
+                    return rv;
+                }
+            }
+        }
+        // move source file to destination file
         copyOK = MoveFile(filePath.get(), destPath.get());
     
+        if (!backup.IsEmpty())
+        {
+            if (copyOK) 
+            {
+                // remove the backup copy.
+                (void) remove(backup.get());
+            }
+            else
+            {
+                // restore backup
+                int backupOk = MoveFile(backup.get(), destPath.get());
+                NS_ASSERTION(backupOk, "move backup failed");
+            }
+        }
+    }        
     if (!copyOK)  // CopyFile and MoveFile returns non-zero if succeeds (backward if you ask me).
         rv = ConvertWinError(GetLastError());
 
@@ -999,8 +1102,6 @@ nsLocalFile::CopyMove(nsIFile *aParentDir, const nsACString &newName, PRBool fol
         if (newName.IsEmpty())
             return NS_ERROR_INVALID_ARG;
 
-        move = PR_TRUE;
-        
         rv = GetParent(getter_AddRefs(newParentDir));
         if (NS_FAILED(rv))
             return rv;
@@ -1012,7 +1113,7 @@ nsLocalFile::CopyMove(nsIFile *aParentDir, const nsACString &newName, PRBool fol
     // make sure it exists and is a directory.  Create it if not there.
     PRBool exists;
     newParentDir->Exists(&exists);
-    if (exists == PR_FALSE)
+    if (!exists)
     {
         rv = newParentDir->Create(DIRECTORY_TYPE, 0644);  // TODO, what permissions should we use
         if (NS_FAILED(rv))
@@ -1103,48 +1204,74 @@ nsLocalFile::CopyMove(nsIFile *aParentDir, const nsACString &newName, PRBool fol
             return rv;
 
         allocatedNewName.Truncate();
+        
+        // check if the destination directory already exists
+        target->Exists(&exists);
+        if (!exists)
+        {
+            // if the destination directory cannot be created, return an error
+            rv = target->Create(DIRECTORY_TYPE, 0644);  // TODO, what permissions should we use
+            if (NS_FAILED(rv))
+                return rv;
+        }
+        else
+        {
+            // check if the destination directory is writable and empty
+            PRBool isWritable;
 
-        target->Create(DIRECTORY_TYPE, 0644);  // TODO, what permissions should we use
-        if (NS_FAILED(rv))
+            target->IsWritable(&isWritable);
+            if (!isWritable)
+                return NS_ERROR_FILE_ACCESS_DENIED;
+
+            nsCOMPtr<nsISimpleEnumerator> targetIterator;
+            rv = target->GetDirectoryEntries(getter_AddRefs(targetIterator));
+
+            PRBool more;
+            targetIterator->HasMoreElements(&more);
+            // return error if target directory is not empty
+            if (more)
+                return NS_ERROR_FILE_DIR_NOT_EMPTY;
+        }
+
+        nsDirEnumerator dirEnum;
+
+        rv = dirEnum.Init(this);
+        if (NS_FAILED(rv)) {
+            NS_WARNING("dirEnum initalization failed");
             return rv;
-        
-        nsDirEnumerator* dirEnum = new nsDirEnumerator();
-        if (!dirEnum)
-            return NS_ERROR_OUT_OF_MEMORY;
-        
-        rv = dirEnum->Init(this);
-
-        nsCOMPtr<nsISimpleEnumerator> iterator = do_QueryInterface(dirEnum);
+        }
 
         PRBool more;
-        iterator->HasMoreElements(&more);
-        while (more)
+        while (NS_SUCCEEDED(dirEnum.HasMoreElements(&more)) && more)
         {
             nsCOMPtr<nsISupports> item;
             nsCOMPtr<nsIFile> file;
-            iterator->GetNext(getter_AddRefs(item));
+            dirEnum.GetNext(getter_AddRefs(item));
             file = do_QueryInterface(item);
-            PRBool isDir, isLink;
+            if (file)
+            {    
+                PRBool isDir, isLink;
             
-            file->IsDirectory(&isDir);
-            file->IsSymlink(&isLink);
-
-            if (move)
-            {
-                if (followSymlinks)
-                    rv = NS_ERROR_FAILURE;
-                else
-                    rv = file->MoveToNative(target, nsCString());
-            }
-            else
-            {   
-                if (followSymlinks)
-                    rv = file->CopyToFollowingLinksNative(target, nsCString());
-                else
-                    rv = file->CopyToNative(target, nsCString());
-            }
+                file->IsDirectory(&isDir);
+                file->IsSymlink(&isLink);
+                
+                if (move)
+                {
+                    if (followSymlinks)
+                        return NS_ERROR_FAILURE;
                     
-            iterator->HasMoreElements(&more);
+                    rv = file->MoveToNative(target, nsCString());
+                    NS_ENSURE_SUCCESS(rv,rv);
+                }
+                else
+                {   
+                    if (followSymlinks)
+                        rv = file->CopyToFollowingLinksNative(target, nsCString());
+                    else
+                        rv = file->CopyToNative(target, nsCString());
+                    NS_ENSURE_SUCCESS(rv,rv);
+                }
+            }
         }
         // we've finished moving all the children of this directory
         // in the new directory.  so now delete the directory
@@ -1153,7 +1280,10 @@ nsLocalFile::CopyMove(nsIFile *aParentDir, const nsACString &newName, PRBool fol
         // we've already moved the children of the current folder 
         // to the new location.  nothing should be left in the folder.
         if (move)
+        {
           rv = Remove(PR_FALSE /* recursive */);
+          NS_ENSURE_SUCCESS(rv,rv);
+        }
     }
     
 
@@ -1230,46 +1360,78 @@ nsLocalFile::Load(PRLibrary * *_retval)
 NS_IMETHODIMP  
 nsLocalFile::Remove(PRBool recursive)
 {
-    PRBool isDir;
-    
-    nsresult rv = IsDirectory(&isDir);
-    if (NS_FAILED(rv))
-        return rv;
+    nsresult rv;
+    PRBool isDir, isLink;
+    nsXPIDLCString buf;
+    const char *filePath;
+
+    // NOTE:
+    //
+    // if the working path points to a shortcut, then we will only
+    // delete the shortcut itself.  even if the shortcut points to
+    // a directory, we will not recurse into that directory or 
+    // delete that directory itself.  likewise, if the shortcut
+    // points to a normal file, we will not delete the real file.
+    // this is done to be consistent with the other platforms that
+    // behave this way.  we do this even if the followLinks attribute
+    // is set to true.  this helps protect against misuse that could
+    // lead to security bugs (e.g., bug 210588).
+    //
+    // in the case of non-terminal shortcuts, those are all followed
+    // unconditionally.  this is done because 1) we only delete 
+    // terminal nodes and possibly their children, and 2) the remove
+    // and rmdir CRT calls don't know how to handle shortcuts.
+
+    IsSymlink(&isLink);
+    if (isLink)
+    {
+        isDir = PR_FALSE;
+        // resolve non-terminal nodes only.
+        rv = ResolvePath(mWorkingPath.get(), PR_FALSE, getter_Copies(buf));
+        if (NS_FAILED(rv))
+            return rv;
+        filePath = buf.get();
+    }
+    else
+    {
+        rv = IsDirectory(&isDir);
+        if (NS_FAILED(rv))
+            return rv;
+        // in this case, it doesn't matter that terminal nodes were
+        // resolved, so we can safely leverage mResolvedPath.
+        filePath = mResolvedPath.get();
+    }
 
-    const char *filePath = mResolvedPath.get();
- 
     if (isDir)
     {
         if (recursive)
         {
-            nsDirEnumerator* dirEnum = new nsDirEnumerator();
-            if (dirEnum == nsnull)
-                return NS_ERROR_OUT_OF_MEMORY;
-        
-            rv = dirEnum->Init(this);
+            nsDirEnumerator dirEnum;
+
+            rv = dirEnum.Init(this);
+            if (NS_FAILED(rv))
+                return rv;
 
-            nsCOMPtr<nsISimpleEnumerator> iterator = do_QueryInterface(dirEnum);
-        
             PRBool more;
-            iterator->HasMoreElements(&more);
-            while (more)
+            while (NS_SUCCEEDED(dirEnum.HasMoreElements(&more)) && more)
             {
                 nsCOMPtr<nsISupports> item;
-                nsCOMPtr<nsIFile> file;
-                iterator->GetNext(getter_AddRefs(item));
-                file = do_QueryInterface(item);
-    
-                file->Remove(recursive);
-                
-                iterator->HasMoreElements(&more);
+                dirEnum.GetNext(getter_AddRefs(item));
+                nsCOMPtr<nsIFile> file = do_QueryInterface(item);
+                if (file)
+                    file->Remove(recursive);
             }
         }
-        rv = rmdir(filePath);  // todo: save return value?
+        rv = rmdir(filePath);
     }
     else
     {
-        rv = remove(filePath); // todo: save return value?
+        rv = remove(filePath);
     }
+
+    // fixup error code if necessary...
+    if (rv == -1)
+        rv = NS_ERROR_FAILURE;
     
     MakeDirty();
     return rv;
@@ -1715,16 +1877,15 @@ nsLocalFile::IsExecutable(PRBool *_retval)
 NS_IMETHODIMP  
 nsLocalFile::IsDirectory(PRBool *_retval)
 {
-    NS_ENSURE_ARG(_retval);
-    *_retval = PR_FALSE;
+    NS_PRECONDITION(_retval, "null pointer");
 
     nsresult rv = ResolveAndStat(PR_TRUE);
-    
-    if (NS_FAILED(rv))
+    if (NS_FAILED(rv)) {
+        *_retval = PR_FALSE;
         return rv;
+    }
 
     *_retval = (mFileInfo64.type == PR_FILE_DIRECTORY); 
-
     return NS_OK;
 }
 
@@ -1753,9 +1914,18 @@ nsLocalFile::IsHidden(PRBool *_retval)
     
     if (NS_FAILED(rv))
         return rv;
-    
-    const char *workingFilePath = mWorkingPath.get();
-    DWORD word = GetFileAttributes(workingFilePath);
+
+    DWORD word;
+    if (mFollowSymlinks)
+    {
+        const char *resolvedFilePath = mResolvedPath.get();
+        word = GetFileAttributes(resolvedFilePath);
+    }
+    else
+    {
+        const char *workingFilePath = mWorkingPath.get();
+        word = GetFileAttributes(workingFilePath);
+    }
 
     *_retval =  ((word & FILE_ATTRIBUTE_HIDDEN)  != 0); 
 
@@ -1765,22 +1935,15 @@ nsLocalFile::IsHidden(PRBool *_retval)
 NS_IMETHODIMP  
 nsLocalFile::IsSymlink(PRBool *_retval)
 {
-    NS_ENSURE_ARG(_retval);
-    *_retval = PR_FALSE;
+    NS_PRECONDITION(_retval, "null pointer");
 
-    nsCAutoString path;
-    int   pathLen;
-    
-    GetNativePath(path);
-    pathLen = path.Length();
-    
-    const char* leaf = path.get() + pathLen - 4;
-    
-    if ( (strcmp(leaf, ".lnk") == 0)) 
-    {
-        *_retval = PR_TRUE;
+    PRUint32 len = mWorkingPath.Length();
+    if (len < 4)
+        *_retval = PR_FALSE;
+    else {
+        const char* leaf = mWorkingPath.get() + len - 4;
+        *_retval = (strnicmp(leaf, ".lnk", 4) == 0);
     }
-    
     return NS_OK;
 }
 
@@ -1831,7 +1994,7 @@ nsLocalFile::Contains(nsIFile *inFile, PRBool recur, PRBool *_retval)
     if ( NS_FAILED(inFile->GetNativeTarget(inFilePath)))
         inFile->GetNativePath(inFilePath);
 
-    if ( strncmp( myFilePath.get(), inFilePath.get(), myFilePathLen) == 0)
+    if ( strnicmp( myFilePath.get(), inFilePath.get(), myFilePathLen) == 0)
     {
         // now make sure that the |inFile|'s path has a trailing
         // separator.
@@ -2156,61 +2319,11 @@ NS_NewNativeLocalFile(const nsACString &path, PRBool followLinks, nsILocalFile*
 // UCS2 interface
 //-----------------------------------------------------------------------------
 
-static nsresult UCS2toFS(const PRUnichar *aBuffer, char **aResult)
-{
-    NS_ENSURE_ARG_POINTER(aBuffer);
-
-    // includes null termination
-    size_t chars = ::WideCharToMultiByte(CP_ACP, 0,
-                                         aBuffer, -1,
-                                         NULL, 0, NULL, NULL);
-    if (chars == 0)
-        return NS_ERROR_FAILURE;
-    
-    *aResult = (char*)nsMemory::Alloc(chars * sizeof(char));
-    if (!*aResult)
-        return NS_ERROR_OUT_OF_MEMORY;
-    
-    // default "defaultChar" is '?', which is an illegal character on windows file system.
-    // That will cause file uncreatable. Change it to '_'
-    const char defaultChar = '_';
-    chars = ::WideCharToMultiByte(CP_ACP, 0,
-                                  aBuffer, -1,
-                                  *aResult, chars, &defaultChar, NULL);
-    if (chars == 0)
-        return NS_ERROR_FAILURE;
-    
-    return NS_OK;
-}
-
-static nsresult FStoUCS2(const char* aBuffer, PRUnichar **aResult)
-{
-    NS_ENSURE_ARG_POINTER(aBuffer);
-
-    // includes null termination
-    size_t chars = ::MultiByteToWideChar(CP_ACP, 0, aBuffer, -1, NULL, 0);
-
-    if (chars == 0)
-        return NS_ERROR_FAILURE;
-    
-    *aResult = (PRUnichar*)nsMemory::Alloc(chars * sizeof(PRUnichar));
-    if (!*aResult)
-        return NS_ERROR_OUT_OF_MEMORY;
-    
-    chars = ::MultiByteToWideChar(CP_ACP, 0,
-                                  aBuffer, -1,
-                                  *aResult, chars);
-    if (chars == 0)
-        return NS_ERROR_FAILURE;
-    
-    return NS_OK;
-}
-
 nsresult  
 nsLocalFile::InitWithPath(const nsAString &filePath)
 {
-    nsXPIDLCString tmp;
-    nsresult rv = UCS2toFS(PromiseFlatString(filePath).get(), getter_Copies(tmp));
+    nsCAutoString tmp;
+    nsresult rv = NS_CopyUnicodeToNative(filePath, tmp);
     if (NS_SUCCEEDED(rv))
         return InitWithNativePath(tmp);
     
@@ -2220,8 +2333,8 @@ nsLocalFile::InitWithPath(const nsAString &filePath)
 nsresult  
 nsLocalFile::Append(const nsAString &node)
 {
-    nsXPIDLCString tmp;
-    nsresult rv = UCS2toFS(PromiseFlatString(node).get(), getter_Copies(tmp));
+    nsCAutoString tmp;
+    nsresult rv = NS_CopyUnicodeToNative(node, tmp);
     if (NS_SUCCEEDED(rv))
         return AppendNative(tmp);
     
@@ -2231,8 +2344,8 @@ nsLocalFile::Append(const nsAString &node)
 nsresult  
 nsLocalFile::AppendRelativePath(const nsAString &node)
 {
-    nsXPIDLCString tmp;
-    nsresult rv = UCS2toFS(PromiseFlatString(node).get(), getter_Copies(tmp));
+    nsCAutoString tmp;
+    nsresult rv = NS_CopyUnicodeToNative(node, tmp);
     if (NS_SUCCEEDED(rv))
         return AppendRelativeNativePath(tmp);
     return rv;
@@ -2243,12 +2356,8 @@ nsLocalFile::GetLeafName(nsAString &aLeafName)
 {
     nsCAutoString tmp;
     nsresult rv = GetNativeLeafName(tmp);
-    if (NS_SUCCEEDED(rv)) {
-        nsXPIDLString ucsBuf;
-        rv = FStoUCS2(tmp.get(), getter_Copies(ucsBuf));
-        if (NS_SUCCEEDED(rv))
-            aLeafName = ucsBuf;
-    }
+    if (NS_SUCCEEDED(rv))
+        rv = NS_CopyNativeToUnicode(tmp, aLeafName);
     
     return rv;
 }
@@ -2256,8 +2365,8 @@ nsLocalFile::GetLeafName(nsAString &aLeafName)
 nsresult  
 nsLocalFile::SetLeafName(const nsAString &aLeafName)
 {
-    nsXPIDLCString tmp;
-    nsresult rv = UCS2toFS(PromiseFlatString(aLeafName).get(), getter_Copies(tmp));
+    nsCAutoString tmp;
+    nsresult rv = NS_CopyUnicodeToNative(aLeafName, tmp);
     if (NS_SUCCEEDED(rv))
         return SetNativeLeafName(tmp);
     
@@ -2267,11 +2376,7 @@ nsLocalFile::SetLeafName(const nsAString &aLeafName)
 nsresult  
 nsLocalFile::GetPath(nsAString &_retval)
 {
-    nsXPIDLString ucsBuf;
-    nsresult rv = FStoUCS2(mWorkingPath.get(), getter_Copies(ucsBuf));
-    if (NS_SUCCEEDED(rv))
-        _retval = ucsBuf;
-    return rv;
+    return NS_CopyNativeToUnicode(mWorkingPath, _retval);
 }
 
 nsresult  
@@ -2280,8 +2385,8 @@ nsLocalFile::CopyTo(nsIFile *newParentDir, const nsAString &newName)
     if (newName.IsEmpty())
         return CopyToNative(newParentDir, nsCString());
     
-    nsXPIDLCString tmp;
-    nsresult rv = UCS2toFS(PromiseFlatString(newName).get(), getter_Copies(tmp));
+    nsCAutoString tmp;
+    nsresult rv = NS_CopyUnicodeToNative(newName, tmp);
     if (NS_SUCCEEDED(rv))
         return CopyToNative(newParentDir, tmp);
     
@@ -2294,8 +2399,8 @@ nsLocalFile::CopyToFollowingLinks(nsIFile *newParentDir, const nsAString &newNam
     if (newName.IsEmpty())
         return CopyToFollowingLinksNative(newParentDir, nsCString());
     
-    nsXPIDLCString tmp;
-    nsresult rv = UCS2toFS(PromiseFlatString(newName).get(), getter_Copies(tmp));
+    nsCAutoString tmp;
+    nsresult rv = NS_CopyUnicodeToNative(newName, tmp);
     if (NS_SUCCEEDED(rv))
         return CopyToFollowingLinksNative(newParentDir, tmp);
     
@@ -2308,8 +2413,8 @@ nsLocalFile::MoveTo(nsIFile *newParentDir, const nsAString &newName)
     if (newName.IsEmpty())
         return MoveToNative(newParentDir, nsCString());
     
-    nsXPIDLCString tmp;
-    nsresult rv = UCS2toFS(PromiseFlatString(newName).get(), getter_Copies(tmp));
+    nsCAutoString tmp;
+    nsresult rv = NS_CopyUnicodeToNative(newName, tmp);
     if (NS_SUCCEEDED(rv))
         return MoveToNative(newParentDir, tmp);
 
@@ -2321,12 +2426,8 @@ nsLocalFile::GetTarget(nsAString &_retval)
 {
     nsCAutoString tmp;
     nsresult rv = GetNativeTarget(tmp);
-    if (NS_SUCCEEDED(rv)) {
-        nsXPIDLString ucsBuf;
-        rv = FStoUCS2(tmp.get(), getter_Copies(ucsBuf));
-        if (NS_SUCCEEDED(rv))
-            _retval = ucsBuf;
-    }
+    if (NS_SUCCEEDED(rv))
+        rv = NS_CopyNativeToUnicode(tmp, _retval);
     
     return rv;
 }
@@ -2334,8 +2435,8 @@ nsLocalFile::GetTarget(nsAString &_retval)
 nsresult 
 NS_NewLocalFile(const nsAString &path, PRBool followLinks, nsILocalFile* *result)
 {
-    nsXPIDLCString tmp;
-    nsresult rv = UCS2toFS(PromiseFlatString(path).get(), getter_Copies(tmp));
+    nsCAutoString tmp;
+    nsresult rv = NS_CopyUnicodeToNative(path, tmp);
     if (NS_SUCCEEDED(rv))
         return NS_NewNativeLocalFile(tmp, followLinks, result);
 
